@@ -20,6 +20,7 @@
 #include <gnome.h>
 
 #include "about.h"
+#include "config_common.h"
 
 void
 about_gtktalog ()
@@ -28,10 +29,12 @@ about_gtktalog ()
   const gchar *authors[3] = { "Mathieu VILLEGAS", "Yves METTIER", NULL };
 
   aboutbox = gnome_about_new (PACKAGE, VERSION,
-			      "Copyright (C) 1999-2000  Mathieu VILLEGAS - Yves Mettier",
+			      "Copyright (C) 1999-2001  Mathieu VILLEGAS - Yves Mettier",
 			      authors,
-			      "http://gtktalog.sourceforge.net\n",
+			      GTKTALOGWEB "\n",
 			      DATA_DIR "/gtktalog/icons/logo.xpm");
-
+  gtk_window_set_modal (GTK_WINDOW (aboutbox), TRUE);
+  gtk_window_set_transient_for (GTK_WINDOW (aboutbox),
+				GTK_WINDOW (main_window));
   gtk_widget_show (aboutbox);
 }
