@@ -18,6 +18,10 @@
 
 #define _USE_BSD
 #include <config.h>
+#if defined(OS_FREEBSD) || defined(OS_NETBSD) || defined(OS_OPENBSD)
+#    include <sys/param.h>
+#    include <sys/mount.h>
+#endif
 #include <gnome.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -48,8 +52,8 @@
 #define READ   0
 #define WRITE 1
 
-static char *local_foldername=NULL;
-static char *local_diskname=NULL;
+static char *local_foldername = NULL;
+static char *local_diskname = NULL;
 static gboolean flag_to_umount;
 static gboolean flag_to_mount;
 static gint added_disk_is_ok;
@@ -63,6 +67,12 @@ static gboolean vfs_scan_nok;
 
 static GtkWidget *stop_window = NULL;
 
+static gint current_progressbar;
+
+#ifdef OS_PPC_DARWIN
+static gchar *mosx_cd_mount_point = NULL;
+#endif
+
 gint
 spawn_proc (gchar * path, gint narg, gchar ** arg)
 {
@@ -70,11 +80,11 @@ spawn_proc (gchar * path, gint narg, gchar ** arg)
   int i, pid;
   GString *gstr = g_string_new (arg[0]);
 
+
   if (test_plugin_existence (gstr))
     {
       g_string_free (gstr, TRUE);
       pid = fork ();
-
       if (pid != 0)
 	return pid;
 
@@ -89,6 +99,7 @@ spawn_proc (gchar * path, gint narg, gchar ** arg)
       close (1);
       close (2);
 #endif
+
       return execvp (arg2[0], arg2);
     }
   else
@@ -107,6 +118,7 @@ du_s (gchar * path)
   gchar *b2;
   ssize_t len;
   int pid;
+
   if (pipe (p) == 0)
     {
       if ((pid = fork ()) == 0)
@@ -119,7 +131,7 @@ du_s (gchar * path)
 	  argsdu[0] = g_strdup ("du");
 #if defined(OS_LINUX)
 	  argsdu[1] = g_strdup ("-sb");
-#elif defined(OS_FREEBSD) || defined(OS_NETBSD)
+#elif defined(OS_FREEBSD) || defined(OS_NETBSD) || defined(OS_OPENBSD) || defined(OS_PPC_DARWIN)
 	  argsdu[1] = g_strdup ("-s");
 #endif
 	  argsdu[2] = path;
@@ -142,7 +154,7 @@ du_s (gchar * path)
 	  /* FIXME: STDIN should be reopened to the real STDIN. */
 	}
     }
-  return (atoi (buffer));
+  return (atol (buffer));
 }
 
 void
@@ -152,7 +164,7 @@ init_message_to_display ()
 }
 
 void
-display_message_if_necessary ()
+display_message_if_necessary (void)
 {
   if (is_message_to_display)
     {
@@ -173,15 +185,35 @@ set_message_to_display (GString * message)
   is_message_to_display = TRUE;
 }
 
-gboolean eject_disk (CONFIG * my_config)
+gboolean
+eject_disk (CONFIG * my_config)
 {
   int status, pid;
-  char *args[2];
+  char *args[3];
+
+  if (my_config->use_automount)
+    {
+#ifdef OS_LINUX
+      if (!my_config->use_supermount)
+#endif
+	return (TRUE);
+    }
+
 
   seteuid (getuid ());
+  if (my_config->use_sudo)
+    {
+  args[0] = my_config->sudo;
+  args[1] = my_config->eject_prog->str;
+  args[2] = my_config->mount_device->str;
+  pid = spawn_proc (NULL, 3, args);
+    }
+  else
+    {
   args[0] = my_config->eject_prog->str;
-  args[1] = escapespaces (my_config->mount_point->str);
+  args[1] = my_config->mount_device->str;
   pid = spawn_proc (NULL, 2, args);
+  }
   if (pid == -1)
     {
       set_message_to_display (g_string_new
@@ -205,20 +237,33 @@ gboolean eject_disk (CONFIG * my_config)
   return (TRUE);
 }
 
-gboolean mount_disk (CONFIG * my_config)
+gboolean
+mount_disk (CONFIG * my_config)
 {
-#if defined(OS_LINUX)
+#if defined(OS_LINUX) || defined(OS_NETBSD) || defined(OS_OPENBSD)
   int pid, status;
-  char *args[2];
+  char *args[3];
+  int nargs;
   int tries = my_config->mount_retry;
 
   seteuid (getuid ());
+  if (my_config->use_sudo)
+    {
+  args[0] = my_config->sudo;
+  args[1] = my_config->mount->str;
+  args[2] = my_config->mount_point->str;
+  nargs = 3;
+    }
+  else
+    {
   args[0] = my_config->mount->str;
-  args[1] = escapespaces (my_config->mount_point->str);
+  args[1] = my_config->mount_point->str;
+      nargs = 2;
+    }
 
   while (tries >= 0)
     {
-      pid = spawn_proc (NULL, 2, args);
+      pid = spawn_proc (NULL, nargs, args);
       if (pid == -1)
 	{
 	  set_message_to_display (g_string_new
@@ -245,7 +290,7 @@ gboolean mount_disk (CONFIG * my_config)
 
   set_message_to_display (g_string_new (_("Mount failed! Disk in use?")));
   return (FALSE);
-#elif defined(OS_FREEBSD) || defined(OS_NETBSD)
+#elif defined(OS_FREEBSD)
   int status;
   struct iso_args args;
 
@@ -259,19 +304,41 @@ gboolean mount_disk (CONFIG * my_config)
     return (TRUE);
   else
     return (FALSE);
+#elif defined(OS_PPC_DARWIN)
+  /*
+   * with MacOS X, CD are mounted automatically so, 
+   * if mosx_get_cd_mount_point() retuns NULL, no CD 
+   * is available or an error has occured
+   */
+
+  return (mosx_get_cd_mount_point (FALSE) == NULL ? FALSE : TRUE);
+
 #endif
 }
 
-gboolean umount_disk (CONFIG * my_config)
+gboolean
+umount_disk (CONFIG * my_config)
 {
-#if defined(OS_LINUX)
+#if defined(OS_LINUX) || defined(OS_NETBSD) || defined(OS_OPENBSD)
   int pid, status;
-  char *args[2];
+  int nargs;
+  char *args[3];
 
   seteuid (getuid ());
+  if (my_config->use_sudo)
+    {
+  args[0] = my_config->sudo;
+  args[1] = my_config->umount->str;
+  args[2] = my_config->mount_point->str;
+  nargs = 3;
+    }
+  else
+    {
   args[0] = my_config->umount->str;
-  args[1] = escapespaces (my_config->mount_point->str);
-  pid = spawn_proc (NULL, 2, args);
+  args[1] = my_config->mount_point->str;
+      nargs = 2;
+    }
+  pid = spawn_proc (NULL, nargs, args);
   if (pid == -1)
     {
       set_message_to_display (g_string_new
@@ -294,7 +361,7 @@ gboolean umount_disk (CONFIG * my_config)
       return (FALSE);
     }
   return (TRUE);
-#elif defined(OS_FREEBSD) || defined(OS_NETBSD)
+#elif defined(OS_FREEBSD)
   int status;
 
   status = unmount (my_config->mount_point->str, NULL);
@@ -302,24 +369,34 @@ gboolean umount_disk (CONFIG * my_config)
     return (TRUE);
   else
     return (FALSE);
+#elif defined(OS_PPC_DARWIN)
+  set_message_to_display (g_string_new (_("Umount failed! Disk in use?")));
+  return (FALSE);
 #endif
 }
 
 void
-umount_and_eject_if_necessary ()
+umount_and_eject_if_necessary (void)
 {
   if (flag_to_umount == TRUE)
     {
       /* umount the drive and eject it */
-      if (my_config->eject_disk == TRUE)
-	{
 	  if ((my_config->use_supermount == TRUE)
 	      || (my_config->use_automount == TRUE))
+	{
+	  if (my_config->eject_disk == TRUE)
+	    {
 	    eject_disk (my_config);
+	    }
+	}
 	  else if (umount_disk (my_config) == TRUE)
+	{
+	  if (my_config->eject_disk == TRUE)
+	    {
 	    eject_disk (my_config);
 	}
     }
+    }
 
 }
 
@@ -335,33 +412,51 @@ do_not_forget_to_mount (gboolean gb)
   flag_to_mount = gb;
 }
 
+void
+reset_added_disk (void)
+{
+  added_disk_is_ok = ADDED_DISK_UNDEF;
+}
+
 void
 set_added_disk (gint gb)
 {
-  if ((gb == ADDED_DISK_DELETE) || (gb == ADDED_DISK_STOP)
-      || (gb == ADDED_DISK_UNDEF) || (added_disk_is_ok == ADDED_DISK_UNDEF))
+  if ((added_disk_is_ok != ADDED_DISK_NOK)
+      && ((gb == ADDED_DISK_DELETE) || (gb == ADDED_DISK_STOP)
+	  || (gb == ADDED_DISK_UNDEF)
+	  || (added_disk_is_ok == ADDED_DISK_UNDEF)))
     {
       added_disk_is_ok = gb;
     }
 }
 
+gboolean
+test_abort_scan (void)
+{
+  if ((added_disk_is_ok == ADDED_DISK_DELETE)
+      || (added_disk_is_ok == ADDED_DISK_STOP))
+    return (TRUE);
+  return (FALSE);
+
+}
+
 void
 init_vfs_scan_status (void)
 {
   vfs_scan_nok = FALSE;
 }
 
-static void
-destroy_window (GtkWidget * w, GdkEventKey * event, gpointer data)
-{
-  gtk_widget_destroy (w);
-}
-
-gint my_timeout (gpointer data)
+gint
+my_timeout (gpointer data)
 {
   FOLDER *racine = data;
+  gint i;
 /*  FILE_DATA *fd;*/
   display_message_if_necessary ();
+  if (progress_needReset ())
+    {
+      progress_reset ();
+    }
   if ((get_thread_status () == THREAD_SCAN_RUNNING_ENDED)
       && (added_disk_is_ok != ADDED_DISK_UNDEF))
     {
@@ -380,39 +475,74 @@ gint my_timeout (gpointer data)
 	}
       else if (added_disk_is_ok == ADDED_DISK_DELETE)
 	{
-	  suppress_dir (local_gn, FALSE);
+	  if (g_node_n_nodes (local_gn, G_TRAVERSE_ALL) <= 1)
+	    {
+	      FILE_DATA *fd = local_gn->data;
+	      for (i = 0; i < racine->datas->len; i++)
+		{
+		  fd = g_ptr_array_index (racine->datas, i);
+		  fd->id = i;
+		}
+	      g_ptr_array_index (racine->datas, fd->id) = NULL;
+	      g_free (fd);
+	      while (g_ptr_array_remove_fast (racine->datas, NULL));
+	      for (i = 0; i < racine->datas->len; i++)
+		{
+		  fd = g_ptr_array_index (racine->datas, i);
+		  fd->id = i;
+		}
+	      g_node_unlink (local_gn);
+	      g_node_destroy (local_gn);
+
+	    }
+	  else
+	    {
+	      suppress_dir (local_gn, FALSE);
+	    }
 	}
       if (stop_window)
 	{
+	  progress_setCurrent (progress_getTotal_m (GNOMEAPPBAR_SCAN),progress_getTotal_b (GNOMEAPPBAR_SCAN),
+			       GNOMEAPPBAR_SCAN);
+	  progress_timeout (GINT_TO_POINTER (GNOMEAPPBAR_SCAN));
+	  current_progressbar = GNOMEAPPBAR_MAIN;
+	  progress_setGnomeAppBar (NULL, GNOMEAPPBAR_SCAN, FALSE);
 	  gtk_widget_destroy (stop_window);
 	  stop_window = NULL;
 	}
-      if (local_diskname) {
-	g_free (local_diskname);
-	local_diskname = NULL;
+      if (local_diskname)
+	{
+	  g_free (local_diskname);
+	  local_diskname = NULL;
 	}
-      if (local_foldername) {
-	g_free (local_foldername);
-	local_foldername = NULL;
+      if (local_foldername)
+	{
+	  g_free (local_foldername);
+	  local_foldername = NULL;
 	}
 /************** If umount and eject have to be in the main thread, remove the comments ********/
 //umount_and_eject_if_necessary();
 /*********************************************************************************************/
-      progress_setCurrent (100, GNOMEAPPBAR_MAIN);
-      progress_timeout (GINT_TO_POINTER (GNOMEAPPBAR_MAIN));
+      progress_setCurrent (progress_getTotal_b (current_progressbar),progress_getTotal_b (current_progressbar),
+			   current_progressbar);
+      progress_timeout (GINT_TO_POINTER (current_progressbar));
 
-      reinit_thread_status (GNOMEAPPBAR_MAIN);
+      reinit_thread_status ();
+      progress_addCurrent ( 1,
+			   GNOMEAPPBAR_MAIN);
+      progress_timeout (GINT_TO_POINTER (GNOMEAPPBAR_MAIN));
       if (vfs_scan_nok)
 	{
 	  WARNING_DIALOG (_
 			  ("At least one VFS could not be read successfully"),
 			  main_window);
 	}
+      gtk_ctree_sort_node (GTK_CTREE (racine->ctree), NULL);
 
     }
   else
     {
-      progress_timeout (GINT_TO_POINTER (GNOMEAPPBAR_MAIN));
+      progress_timeout (GINT_TO_POINTER (current_progressbar));
       gtk_timeout_add (PROGRESS_BAR_TIMEOUT, my_timeout, racine);
     }
   return (0);
@@ -421,19 +551,37 @@ gint my_timeout (gpointer data)
 void
 fad_wait_window_cb (GtkWidget * w, gpointer data)
 {
+  progress_addCurrent ( 1,
+		       GNOMEAPPBAR_SCAN);
+  progress_timeout (GINT_TO_POINTER (GNOMEAPPBAR_SCAN));
+  current_progressbar = GNOMEAPPBAR_MAIN;
+  progress_setGnomeAppBar (NULL, GNOMEAPPBAR_SCAN, FALSE);
   set_added_disk (GPOINTER_TO_INT (data));
   gtk_widget_destroy (stop_window);
   stop_window = NULL;
 }
 
+static void
+destroy_fad_window (GtkWidget * w, GdkEventKey * event, gpointer data)
+{
+  progress_addCurrent ( 1,
+		       GNOMEAPPBAR_SCAN);
+  progress_timeout (GINT_TO_POINTER (GNOMEAPPBAR_SCAN));
+  current_progressbar = GNOMEAPPBAR_MAIN;
+  progress_setGnomeAppBar (NULL, GNOMEAPPBAR_SCAN, FALSE);
+
+  gtk_widget_destroy (w);
+  fad_wait_window ();
+}
+
 void
 fad_wait_window (void)
 {
   GtkWidget *label;
+  GtkWidget *progressbar;
 
   /* Window */
-  stop_window = gnome_dialog_new (_("Wait"),
-				  _("Stop"), GNOME_STOCK_BUTTON_CANCEL, NULL);
+  stop_window = gnome_dialog_new (_("Scanning"), _("Stop"), _("Abort"), NULL);
 
   /* FIXME: Set Parent to help the window manager */
   //gnome_dialog_set_parent(GNOME_DIALOG(dlg), GTK_WINDOW(app));
@@ -441,12 +589,31 @@ fad_wait_window (void)
   gtk_window_set_position (GTK_WINDOW (stop_window), GTK_WIN_POS_MOUSE);
 
   /* Label */
-  label = gtk_label_new (_("Stop:   Stop the scan and keep the disk.\n"
-                           "Cancel: Stop the scan and remove the disk."));
+  label = gtk_label_new (_("Scanning..."));
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (stop_window)->vbox), label, TRUE,
 		      TRUE, 0);
   gtk_widget_show (label);
 
+  /* Progress bar */
+  progressbar = gtk_progress_bar_new ();
+  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (stop_window)->vbox), progressbar,
+		      TRUE, TRUE, 0);
+  gtk_widget_show (progressbar);
+
+
+  progress_setGnomeAppBar (progressbar, GNOMEAPPBAR_SCAN, FALSE);
+  progress_set_activity_mode (GNOMEAPPBAR_SCAN, FALSE);
+  progress_setCurrent (progress_getCurrent_m (GNOMEAPPBAR_MAIN),progress_getCurrent_b (GNOMEAPPBAR_MAIN),
+		       GNOMEAPPBAR_SCAN);
+  progress_setTotal (progress_getTotal_m (GNOMEAPPBAR_MAIN), progress_getTotal_b (GNOMEAPPBAR_MAIN), GNOMEAPPBAR_SCAN);
+  progress_setCurrent (progress_getTotal_m (GNOMEAPPBAR_MAIN),progress_getTotal_b (GNOMEAPPBAR_MAIN),
+		       GNOMEAPPBAR_MAIN);
+  current_progressbar = GNOMEAPPBAR_SCAN;
+#if !defined(USE_PTHREADS)
+  my_timeout (my_config->racine);
+//      progress_timeout (GINT_TO_POINTER (current_progressbar));
+#endif
+
   /* Callbacks */
   gnome_dialog_button_connect (GNOME_DIALOG (stop_window), 0,
 			       fad_wait_window_cb,
@@ -455,10 +622,13 @@ fad_wait_window (void)
 			       fad_wait_window_cb,
 			       GINT_TO_POINTER (ADDED_DISK_DELETE));
 
+  gtk_window_set_modal (GTK_WINDOW (stop_window), TRUE);
   gtk_window_set_transient_for (GTK_WINDOW (stop_window),
 				GTK_WINDOW (main_window));
   gtk_signal_connect (GTK_OBJECT (stop_window),
-		      "delete_event", destroy_window, NULL);
+		      "delete_event", destroy_fad_window, NULL);
+  gtk_signal_connect (GTK_OBJECT (stop_window),
+		      "close", destroy_fad_window, NULL);
   gtk_widget_realize (stop_window);
   gtk_widget_show (stop_window);
 
@@ -524,7 +694,6 @@ launch_information_plugin (gchar * file, MIME_EXTENSIONS * m)
 		(gchar **) g_realloc (args, sizeof (gchar *) * (nb_args + 2));
 	      args[nb_args] = g_strdup (file);
 	      args[nb_args + 1] = NULL;
-
 	      execvp (args[0], args);
 	    }
 	  else
@@ -566,6 +735,10 @@ read_information (gchar * file, gchar * mime)
 {
   MIME_EXTENSIONS *m;
   gchar *information_string = NULL;
+#ifdef OS_PPC_DARWIN
+  int file_len;
+#endif
+
   if (mime)
     {
       if ((m = find_extension (mime)))
@@ -584,10 +757,35 @@ read_information (gchar * file, gchar * mime)
 	    }
 	}
     }
+
+  if (information_string)
+    {
+#ifdef OS_PPC_DARWIN
+      /* in MacOS X, the file utility doesn't accept the -b parameter */
+      /* so we remove the file name and ": " from the beginning of information_string */
+      file_len = strlen (file) /*file_len */ ;
+      if (strncmp (information_string, file, file_len) == 0)
+	{
+	  if (strncmp (&information_string[file_len], ": ", 2) == 0)
+	    {
+	      strcpy (information_string, &information_string[file_len + 2]);
+	    }
+	  else
+	    {
+	      strcpy (information_string, &information_string[file_len]);
+	    }
+	}
+#endif
+      /* remove the last char which is a \n */
+      while ((strlen (information_string) > 0)
+	     && (information_string[strlen (information_string) - 1] == '\n'))
+	information_string[strlen (information_string) - 1] = 0;
+    }
   return (information_string);
 }
 
-gboolean is_dir_openable (char *dir_name)
+gboolean
+is_dir_openable (char *dir_name)
 {
   DIR *dir;
 
@@ -615,22 +813,18 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
   GNode *gn;
   GString *my_path;
   gchar *infostring;
-  static gulong totalDataBlocks = 0;
   gchar *mime;
   gpointer scan_it;
 
   if (added_disk_is_ok == ADDED_DISK_DELETE)
-    return (FALSE);
-
-  if (added_disk_is_ok == ADDED_DISK_STOP)
-    return (FALSE);
-
-  if (progress_needReset ())
     {
-      progress_reset ();
-      totalDataBlocks = 0;
+      return (FALSE);
     }
 
+  if (added_disk_is_ok == ADDED_DISK_STOP)
+    {
+      return (FALSE);
+    }
   my_path = g_string_new (path);
 
   my_path = g_string_append_c (my_path, '/');
@@ -639,8 +833,12 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
 
   if ((dir = opendir (my_path->str)) == NULL)
     {
-      /* FIXME: put an English message inside a dialog box */
-      printf ("Can't read the directory: %s", my_path->str);
+      char *tmp;
+      tmp = g_strdup_printf (_("Can't read the directory: %s"), my_path->str);
+      while (is_message_to_display)
+	sleep (1);
+      set_message_to_display (g_string_new (tmp));
+      g_free (tmp);
       return (FALSE);
     }
   while (((fichier = readdir (dir)) != NULL)
@@ -648,14 +846,15 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
 	 && (added_disk_is_ok != ADDED_DISK_STOP))
     {
       file = g_strconcat (my_path->str, fichier->d_name, NULL);
-      if ((strcmp (fichier->d_name, ".") != 0)
-	  && (strcmp (fichier->d_name, "..") != 0))
+      if ((strcmp (fichier->d_name, ".") )
+	  && (strcmp (fichier->d_name, "..") ))
 	{
 
 	  if (lstat (file, &fileinfo) == -1)
 	    {
 	      gn = folder_add (racine, parent, fichier->d_name, NULL,
 			       IS_UNREADABLE, NULL, 0, 0, 0, 0);
+	      /* Progress Bar */
 	    }
 	  /* This is a directory */
 	  else if (S_ISLNK (fileinfo.st_mode))
@@ -663,22 +862,32 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
 	      char tmp[1024];
 	      char tmp2[2048];
 
+	      progress_addCurrent (fileinfo.st_size, current_progressbar);
+
 	      memset (tmp, 0, 1024);
 
 	      if (readlink (file, tmp, 1023) == -1)
 		{
-		  printf
-		    ("Problem with symbolic link %s, we forget it!\n", file);
+		  char *tmp;
+		  tmp =
+		    g_strdup_printf (_
+				     ("Problem with symbolic link %s, we forget it!"),
+				     file);
+		  while (is_message_to_display)
+		    sleep (1);
+		  set_message_to_display (g_string_new (tmp));
+		  g_free (tmp);
 		}
 	      else
 		{
-		  sprintf (tmp2, "%s -> %s", fichier->d_name, tmp);
+		  snprintf (tmp2, 2047, "%s -> %s", fichier->d_name, tmp);
 		  folder_add_link (racine, parent, tmp, tmp2, NULL,
 				   fileinfo.st_size, fileinfo.st_ctime, 0, 0);
 		}
 	    }
 	  else if (S_ISDIR (fileinfo.st_mode))
 	    {
+	      progress_addCurrent (fileinfo.st_size, current_progressbar);
 
 	      if (is_dir_openable (file))
 		{
@@ -693,10 +902,7 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
 	  else
 	    {
 	      /* This is a file */
-
-	      /* Progress Bar */
-	      totalDataBlocks += fileinfo.st_size;
-	      progress_setCurrent (totalDataBlocks, GNOMEAPPBAR_MAIN);
+	      progress_addCurrent (fileinfo.st_size, current_progressbar);
 
 	      mime = NULL;
 	      if (my_config->scan_mime_types)
@@ -746,8 +952,12 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
 		    g_free (mime);
 		}
 	    }
+	} else {
+		  if (lstat (file, &fileinfo) >=0)
+	      progress_addCurrent (fileinfo.st_size, current_progressbar);
+
 	}
-      free (file);
+      g_free (file);
 #if !defined(USE_PTHREADS)
       gui_update ();
 #endif
@@ -762,7 +972,7 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
       file = g_strconcat (path, "/", folder_get_name (gn), NULL);
       if (create_folder_tree (racine, gn, file, getInformation, testvfs))
 	vfs_scan_nok = TRUE;
-      free (file);
+      g_free (file);
       gl = new_dirs;
       new_dirs = g_list_remove_link (new_dirs, new_dirs);
       g_list_free_1 (gl);
@@ -770,8 +980,11 @@ create_folder_tree (FOLDER * racine, GNode * parent, const gchar * path,
   return (vfs_scan_nok);
 }
 
-gboolean check_mounted (CONFIG * my_config)
+gboolean
+check_mounted (CONFIG * my_config)
 {
+
+#ifdef OS_LINUX
   FILE *fp;
   char line[1024];
   char s1[1024];
@@ -779,7 +992,6 @@ gboolean check_mounted (CONFIG * my_config)
   char s3[1024];
   int rc;
 
-#ifdef OS_LINUX
   fp = fopen ("/proc/mounts", "r");
   if (fp == NULL)
     ERROR_DIALOG (_("Unable to open /proc/mounts\n"), main_window);
@@ -795,6 +1007,16 @@ gboolean check_mounted (CONFIG * my_config)
     }
   fclose (fp);
   return FALSE;
+#elif (defined(OS_PPC_DARWIN))
+
+  /*
+   * with MacOS X, CD are mounted automatically so, 
+   * if mosx_get_cd_mount_point() retuns NULL, no CD 
+   * is available or an error has occured
+   */
+
+  return (mosx_get_cd_mount_point (FALSE) == NULL ? FALSE : TRUE);
+
 #else
 /* FIXME: in many cases, this is a bug as we do not know if the CD is mounted
  * or not.
@@ -806,22 +1028,12 @@ gboolean check_mounted (CONFIG * my_config)
 #endif
 }
 
-void *
-start_thread_scan (void *arg)
+int
+mount_disk_if_necessary (void)
 {
-  FOLDER *racine = arg;
-#if defined(__DEBUG__)
-  GML_log (2, "start_thread_scan()\n");
-#endif
-  init_message_to_display ();
-#if defined(USE_PTHREADS)
-  pthread_detach (pthread_self ());
-#endif
-
-  /* Mount the CD if we are in fastadddisk */
-  flag_to_umount = FALSE;
   if (flag_to_mount == TRUE)
     {
+      flag_to_umount = FALSE;
       if ((my_config->use_supermount == FALSE)
 	  && (my_config->use_automount == FALSE))
 	{
@@ -842,17 +1054,18 @@ start_thread_scan (void *arg)
 		  /* Display Error */
 		  set_message_to_display (g_string_new
 					  (_
-					   ("Could not mount the device. Is the CD player empty?")));
+					   ("Could not mount the device.\n"
+					    "Check there is a CD in the CD player\n"
+					    "If yes, check the mount point (see the preferences)\n"
+					    "Then try again.\n"
+					    "If the problem persists, try to manually mount your CDROM.")));
 		  added_disk_is_ok = ADDED_DISK_NOK;
 		  set_thread_status_to_ended ();
 		  vfs_scan_nok = FALSE;
 #if defined(__DEBUG__)
 		  GML_log (2, "Exiting start_thread_scan(): error\n");
 #endif
-#if defined(USE_PTHREADS)
-		  pthread_exit (NULL);
-#endif
-		  return (NULL);
+		  return (0);
 		}
 	      else
 		flag_to_umount = TRUE;
@@ -861,20 +1074,66 @@ start_thread_scan (void *arg)
       else
 	flag_to_umount = TRUE;
     }
+  return (1);
+}
+
+void *
+start_thread_scan (void *arg)
+{
+  FOLDER *racine = arg;
+  guint l;
+#if defined(USE_PTHREADS)
+  current_progressbar = GNOMEAPPBAR_MAIN;
+  progress_setTotal (0,0, GNOMEAPPBAR_SCAN);
+#else
+  current_progressbar = GNOMEAPPBAR_SCAN;
+#endif
+  progress_setCurrent (0,0, current_progressbar);
+  progress_setTotal (0,0, current_progressbar);
+#if defined(__DEBUG__)
+  GML_log (2, "start_thread_scan()\n");
+#endif
+  init_message_to_display ();
+#if defined(USE_PTHREADS)
+  pthread_detach (pthread_self ());
+#endif
+
+
+  /* Mount the CD if we are in fastadddisk */
+  if (!mount_disk_if_necessary ())
+    {
+#if defined(USE_PTHREADS)
+      pthread_exit (NULL);
+#endif
+      return (NULL);
+    }
 
-  /* Get the size of the disk to init the progress bar */
   if (my_config->use_du_s == TRUE)
-    progress_setTotal (du_s (local_foldername), GNOMEAPPBAR_MAIN);
+    l = du_s (local_foldername);
   else
-    progress_setTotal (getStats (escapespaces (my_config->mount_point->str)),
-		       GNOMEAPPBAR_MAIN);
+    l = getStats (my_config->mount_device->str);
+  progress_setTotal (0,l, current_progressbar);
 
   /* Launch the update of the progress bar */
 /*  progress_createTimeout ();*/
 
-  local_gn =
-    folder_add (racine, racine->tree,
-		local_diskname, local_foldername, IS_DISK, NULL, 0, 0, 0, 0);
+#ifdef OS_PPC_DARWIN
+  if (strcmp (local_foldername, mosx_get_cd_mount_point (FALSE)))
+#else
+  if (strcmp (local_foldername, my_config->mount_point->str))
+#endif
+    {
+      local_gn =
+	folder_add (racine, racine->tree,
+		    local_diskname, local_foldername, IS_DISK, NULL, 0, 0, 0,
+		    0);
+    }
+  else
+    {
+      local_gn =
+	folder_add (racine, racine->tree,
+		    local_diskname, NULL, IS_CD, NULL, 0, 0, 0, 0);
+    }
 #if !defined(USE_PTHREADS)
   gui_update ();
 #endif
@@ -886,7 +1145,6 @@ start_thread_scan (void *arg)
   set_thread_status_to_ended ();
   racine->is_modified = TRUE;
 #if defined(__DEBUG__)
-  GML_dump (GML_tree_gstrings, FALSE);
   GML_log (2, "Exiting start_thread_scan(): OK\n");
 #endif
 #if defined(USE_PTHREADS)
@@ -910,7 +1168,7 @@ escapespaces (gchar * txt)
       n++;
       ptr++;
     }
-  ptr = (gchar *) malloc (sizeof (gchar) * n);
+  ptr = (gchar *) g_malloc (sizeof (gchar) * n);
   ptmp = ptr;
   while (txt[0])
     {
@@ -926,3 +1184,144 @@ escapespaces (gchar * txt)
   ptmp[0] = 0;
   return (ptr);
 }
+
+#ifdef OS_PPC_DARWIN
+gchar *
+mosx_get_cd_mount_point (int force)
+{
+  FILE *fpipe;
+  char buffer[1024], *p = NULL, device[32], dum[64];
+  int unit = -1;
+  int ejectable, writable, typecd, found, newdevice;
+
+  if (force)
+    {
+      if (mosx_cd_mount_point != NULL)
+	{
+	  g_free (mosx_cd_mount_point);
+	  mosx_cd_mount_point = NULL;
+	}
+    }
+
+  if (mosx_cd_mount_point != NULL)
+    return mosx_cd_mount_point;
+
+  fpipe = popen ("ioreg -c IOMedia -w 0", "r");
+  if (fpipe == NULL)
+    {
+      fprintf (stderr, "popen ioreg failed\n");
+      return NULL;
+    }
+
+  found = 0;
+  newdevice = 0;
+  ejectable = writable = typecd = -1;
+
+  while (fgets (buffer, sizeof (buffer), fpipe) != NULL)
+    {
+      if (found)
+	continue;		/* don't know if the pipe can be closed */
+      p = strchr (buffer, '\n');
+      if (p)
+	*p = 0;
+      if ((p = strstr (buffer, "\"BSD Unit\" = ")))
+	{
+	  p += 13;
+	  if (atoi (p) != unit)
+	    {
+	      newdevice = 1;
+	      if (unit == -1)
+		{
+		  unit = atoi (p);
+		  continue;
+		}
+	      if (typecd == 1 || (ejectable == 1 && writable == 0))
+		{
+		  /* a CD-ROM drive is found */
+		  found = 1;
+		}
+	      else
+		{
+		  unit = atoi (p);
+		  typecd = ejectable = writable = -1;
+		  *device = 0;
+		}
+	    }
+	  else
+	    newdevice = 0;
+	}			/* endif BSD Unit */
+      if (newdevice && (p = strstr (buffer, "\"Type\" = ")))
+	{
+	  if (!strncmp (p + 9, "\"CD-ROM\"", 8))
+	    {
+	      typecd = 1;
+	    }
+	}
+      if (newdevice && (p = strstr (buffer, "\"Ejectable\" = ")))
+	{
+	  if (!strncmp (p + 14, "Yes", 3))
+	    ejectable = 1;
+	  else if (!strncmp (p + 14, "No", 2))
+	    ejectable = 0;
+	}
+      if (newdevice && (p = strstr (buffer, "\"Writable\" = ")))
+	{
+	  if (!strncmp (p + 13, "Yes", 3))
+	    writable = 1;
+	  else if (!strncmp (p + 13, "No", 2))
+	    writable = 0;
+	}
+      if (newdevice && (p = strstr (buffer, "\"BSD Name\" = \"")))
+	{
+	  strcpy (device, p + 14);
+	  p = strchr (device, '\"');
+	  if (p)
+	    *p = 0;
+	}
+    }				/* end while */
+  fclose (fpipe);
+
+  if (!(typecd == 1 || (ejectable == 1 && writable == 0)))
+    {
+      return NULL;		/* no CD-ROM found */
+    }
+
+  strcpy (buffer, "df | grep ");
+  strcat (buffer, device);
+  fpipe = popen (buffer, "r");
+  if (fpipe == NULL)
+    {
+      fprintf (stderr, "popen df failed\n");
+      return NULL;
+    }
+
+  found = 0;
+  strcpy (dum, "/dev/");
+  strcat (dum, device);
+  while (fgets (buffer, sizeof (buffer), fpipe) != NULL)
+    {
+      if (found)
+	continue;		/* don't know if the pipe can be closed */
+      p = strchr (buffer, '\n');
+      if (p)
+	*p = 0;
+      if (strncmp (buffer, dum, strlen (dum)) == 0)
+	{
+	  p = strstr (buffer, "/Volumes/");
+	  if (p)
+	    found = 1;
+	}
+    }				/* end while */
+  fclose (fpipe);
+
+  if (found)
+    {
+      mosx_cd_mount_point = g_strdup (p);
+      return mosx_cd_mount_point;
+    }
+  else
+    {
+      return NULL;
+    }
+}
+#endif
