@@ -30,10 +30,13 @@
 #include "exit.h"
 #include "categories.h"
 #include "loadcat.h"
+#include "compare.h"
 
 static GtkWidget *cat_window;
 static GtkWidget *cat_clist;
 static GtkWidget *import_selector;
+static GtkWidget *button_edit;
+static GtkWidget *button_delete;
 static gint selected_row;
 static guint cat_id;
 
@@ -67,25 +70,44 @@ void
 add_the_categorie (GtkWidget * w, FOLDER * racine)
 {
   char *tmp[2] = { "", "0" };
-  GString *category_name;
+  GString *category_name = NULL;
+  gint button;
 
-  gnome_dialog_run_and_close (GNOME_DIALOG (gnome_request_dialog
-					    (FALSE,
-					     _("New category name"),
-					     "",
-					     CATEGORY_MAX_LENGTH,
-					     _set_category_name,
-					     &category_name,
-					     GTK_WINDOW (cat_window))));
+  button = gnome_dialog_run_and_close (GNOME_DIALOG (gnome_request_dialog
+						     (FALSE,
+						      _("New category name"),
+						      "",
+						      CATEGORY_MAX_LENGTH,
+						      _set_category_name,
+						      &category_name,
+						      GTK_WINDOW
+						      (cat_window))));
 
 
-
-  if (category_name)
+  if (button == 0)
     {
-      g_ptr_array_add (racine->categories, category_name);
-      tmp[0] = ((GString *) category_name)->str;
-      gtk_clist_append ((GtkCList *) cat_clist, tmp);
-      racine->is_modified = TRUE;
+      if (category_name
+	  && compare ("^[\t ]*$", category_name->str, COMPARE_REGEX, TRUE))
+	{
+	  if (string_to_index (racine->categories, category_name->str) == 0)
+	    {
+	      g_ptr_array_add (racine->categories, category_name);
+	      tmp[0] = ((GString *) category_name)->str;
+	      gtk_clist_append ((GtkCList *) cat_clist, tmp);
+	      racine->is_modified = TRUE;
+	    }
+	  else
+	    {
+	      WARNING_DIALOG (_
+			      ("A category with the same name already exists. Cancel"),
+			      cat_window);
+	    }
+	}
+
+      else
+	{
+	  WARNING_DIALOG (_("Empty string. Cancel"), cat_window);
+	}
     }
   return;
 }
@@ -93,38 +115,60 @@ add_the_categorie (GtkWidget * w, FOLDER * racine)
 void
 edit_the_categorie (GtkWidget * w, FOLDER * racine)
 {
-  gint i;
+  gint i, j;
   char *tmp[2] = { "", "0" };
-  GString *category_name;
+  GString *category_name = NULL;
+  gint button;
 
   if (selected_row >= 0)
     {
       gtk_clist_get_text ((GtkCList *) cat_clist, selected_row, 0, &(tmp[0]));
       i = string_to_index (racine->categories, tmp[0]) - 1;
 
-      gnome_dialog_run_and_close (GNOME_DIALOG (gnome_request_dialog
-						(FALSE,
-						 _("New category name"),
-						 tmp[0],
-						 CATEGORY_MAX_LENGTH,
-						 _set_category_name,
-						 &category_name,
-						 GTK_WINDOW (cat_window))));
+      button = gnome_dialog_run_and_close (GNOME_DIALOG (gnome_request_dialog
+							 (FALSE,
+							  _
+							  ("New category name"),
+							  tmp[0],
+							  CATEGORY_MAX_LENGTH,
+							  _set_category_name,
+							  &category_name,
+							  GTK_WINDOW
+							  (cat_window))));
 
 
-
-      if (category_name)
+      if (button == 0)
 	{
-	  g_string_free (g_ptr_array_index (racine->categories, i), TRUE);
-	  tmp[0] = ((GString *) category_name)->str;
-	  g_ptr_array_index (racine->categories, i) = g_string_new (tmp[0]);
-	  gtk_clist_set_text ((GtkCList *) cat_clist, selected_row, 0,
-			      tmp[0]);
-	  racine->is_modified = TRUE;
-	}
-      else
-	{
-	  WARNING_DIALOG (_("Empty string. No change"), cat_window);
+	  if (category_name
+	      && compare ("^[\t ]*$", category_name->str, COMPARE_REGEX,
+			  TRUE))
+	    {
+	      j =
+		string_to_index (racine->categories, category_name->str) - 1;
+	      if (j == i)
+		return;
+	      if (j == -1)
+		{
+		  g_string_free (g_ptr_array_index (racine->categories, i),
+				 TRUE);
+		  tmp[0] = ((GString *) category_name)->str;
+		  g_ptr_array_index (racine->categories, i) =
+		    g_string_new (tmp[0]);
+		  gtk_clist_set_text ((GtkCList *) cat_clist, selected_row, 0,
+				      tmp[0]);
+		  racine->is_modified = TRUE;
+		}
+	      else
+		{
+		  WARNING_DIALOG (_
+				  ("A category with the same name already exists. Cancel"),
+				  cat_window);
+		}
+	    }
+	  else
+	    {
+	      WARNING_DIALOG (_("Empty string. No change"), cat_window);
+	    }
 	}
     }
   return;
@@ -213,6 +257,9 @@ import_clicked_ok (GtkWidget * widget, gpointer data)
 void
 import_clicked (GtkWidget * w, gpointer data)
 {
+  GtkAccelGroup *accel_group;
+
+  accel_group = gtk_accel_group_new ();
 
   import_selector = gtk_file_selection_new (_("Select Disk location"));
   gtk_file_selection_set_filename (GTK_FILE_SELECTION (import_selector),
@@ -230,6 +277,11 @@ import_clicked (GtkWidget * w, gpointer data)
 			     GTK_OBJECT (import_selector));
   gtk_window_set_modal (GTK_WINDOW (import_selector), TRUE);
   gtk_widget_show (import_selector);
+  gtk_widget_add_accelerator ((GTK_FILE_SELECTION (import_selector)->
+			       cancel_button), "clicked", accel_group,
+			      GDK_Escape, 0, GTK_ACCEL_VISIBLE);
+
+  gtk_window_add_accel_group (GTK_WINDOW (import_selector), accel_group);
 }
 
 gboolean
@@ -249,8 +301,10 @@ is_category_empty (GNode * gn, gpointer data)
   return (FALSE);
 }
 
+
+
 void
-remove_unused_categories (FOLDER * racine)
+remove_unused_categories (FOLDER * racine, gboolean update_gui)
 {
   gint i, j, id_cat;
   static guint tmpcount;
@@ -277,7 +331,8 @@ remove_unused_categories (FOLDER * racine)
 
       g_string_free (g_ptr_array_index (racine->categories, id_cat), TRUE);
       g_ptr_array_remove_index (racine->categories, id_cat);
-      gtk_clist_remove ((GtkCList *) cat_clist, id_cat);
+      if (update_gui)
+	gtk_clist_remove ((GtkCList *) cat_clist, id_cat);
       for (j = 1; j < racine->datas->len; j++)
 	{
 	  fd = g_ptr_array_index (racine->datas, j);
@@ -295,7 +350,7 @@ remove_unused_categories (FOLDER * racine)
 void
 remove_unused_clicked (GtkWidget * w, FOLDER * racine)
 {
-  remove_unused_categories (racine);
+  remove_unused_categories (racine, TRUE);
   list_folder (racine->tree, TRUE);
 
 
@@ -306,6 +361,8 @@ cat_selected (GtkWidget * clist, gint row, gint column,
 	      GdkEventButton * event, gpointer data)
 {
   selected_row = row;
+  gtk_widget_set_sensitive (GTK_WIDGET (button_edit), TRUE);
+  gtk_widget_set_sensitive (GTK_WIDGET (button_delete), TRUE);
   return;
 }
 
@@ -315,6 +372,8 @@ cat_unselected (GtkWidget * clist, gint row, gint column,
 		GdkEventButton * event, gpointer data)
 {
   selected_row = -1;
+  gtk_widget_set_sensitive (GTK_WIDGET (button_edit), FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (button_delete), FALSE);
   return;
 }
 
@@ -351,15 +410,19 @@ edit_categories (FOLDER * racine)
 {
   GtkWidget *scrolled_window;
   GtkWidget *vbox, *hbox, *vbox1, *hbox1;
-  GtkWidget *button_add, *button_edit, *button_delete;
+  GtkWidget *button_add;
   GtkWidget *button_import, *button_remove_unused, *button_exit;
   gchar *titles[2] = { _("Name"), _("Use") };
   gchar count[] = "      ";
   static guint tmpcount;
   gint i;
+  GtkAccelGroup *accel_group;
+
+  accel_group = gtk_accel_group_new ();
 
   cat_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
+  gtk_window_set_modal (GTK_WINDOW (cat_window), TRUE);
   gtk_window_set_transient_for (GTK_WINDOW (cat_window),
 				GTK_WINDOW (main_window));
 
@@ -461,20 +524,25 @@ edit_categories (FOLDER * racine)
   gtk_widget_show (button_edit);
   gtk_signal_connect (GTK_OBJECT (button_edit), "clicked",
 		      GTK_SIGNAL_FUNC (edit_the_categorie), racine);
-
+  gtk_widget_set_sensitive (GTK_WIDGET (button_edit), FALSE);
 
   button_delete = gnome_stock_or_ordinary_button ("Remove");
   gtk_box_pack_start (GTK_BOX (vbox1), button_delete, TRUE, FALSE, 0);
   gtk_widget_show (button_delete);
   gtk_signal_connect (GTK_OBJECT (button_delete), "clicked",
 		      GTK_SIGNAL_FUNC (del_the_categorie), racine);
+  gtk_widget_set_sensitive (GTK_WIDGET (button_delete), FALSE);
 
 
-  button_exit = gnome_stock_or_ordinary_button ("Quit");
+  button_exit = gnome_stock_or_ordinary_button ("Close");
   gtk_box_pack_start (GTK_BOX (hbox1), button_exit, TRUE, TRUE, 0);
   gtk_widget_show (button_exit);
   gtk_signal_connect (GTK_OBJECT (button_exit), "clicked",
 		      GTK_SIGNAL_FUNC (cancel_clicked), racine);
 
 
+  gtk_widget_add_accelerator (button_exit, "clicked", accel_group,
+			      GDK_Escape, 0, GTK_ACCEL_VISIBLE);
+
+  gtk_window_add_accel_group (GTK_WINDOW (cat_window), accel_group);
 }
