@@ -28,6 +28,10 @@
 #include "config_common.h"
 #include "folder.h"
 #include "io.h"
+#include "fastaddisk.h"
+#include "addisk.h"
+#include "exit.h"
+#include "thread_utils.h"
 
 static GtkWidget *edit_window;
 static GtkWidget *fold_entry;
@@ -46,7 +50,8 @@ struct _gnode_and_string
   gchar *s;
 };
 
-gboolean is_file_with_the_same_name (GNode * gn, gpointer data)
+gboolean
+is_file_with_the_same_name (GNode * gn, gpointer data)
 {
   FILE_DATA *fd;
   struct _gnode_and_string *gns = data;
@@ -135,8 +140,7 @@ ok_clicked (GtkWidget * w, gpointer data)
       if ((tmpgn) && (tmpgn != gn->parent))
 	{
 	  ERROR_DIALOG (_("A file/dir with the same name "
-	                  "already exists in this directory"),
-	                edit_window);
+			  "already exists in this directory"), edit_window);
 	  return;
 	}
     }
@@ -223,9 +227,39 @@ void
 delete_folder (GtkWidget * w, FOLDER * racine)
 {
   GNode *parent;
+  GNode *gn;
+  GList *gl;
+  GList *gl_del = NULL;
+
+  if (yes_no_dialog (_("Deletion confirmation"), _("Really delete?")))
+    return;
+
   parent = racine->selected_folder->parent;
   gtk_clist_freeze (GTK_CLIST (racine->ctree));
-  suppress_dir (racine->selected_folder, TRUE);
+  gl = (GTK_CLIST (clist))->selection;
+  if (g_list_length (gl) > 1)
+    {
+      while (gl)
+	{
+	  gn =
+	    gtk_clist_get_row_data (GTK_CLIST (clist),
+				    GPOINTER_TO_INT (gl->data));
+	  gl_del = g_list_append (gl_del, gn);
+	  gl = g_list_next (gl);
+	}
+      gl = gl_del;
+      while (gl)
+	{
+	  gn = gl->data;
+	  suppress_dir (gn, TRUE);
+	  gl = g_list_next (gl);
+	}
+      g_list_free (gl_del);
+    }
+  else
+    {
+      suppress_dir (racine->selected_folder, TRUE);
+    }
   description_garbage_collection ();
   gtk_clist_thaw (GTK_CLIST (racine->ctree));
   list_folder (parent, TRUE);
@@ -259,7 +293,8 @@ update_category_combo (FOLDER * racine)
 					 (racine->categories, i))->str);
   if (cat_list)
     gtk_combo_set_popdown_strings (GTK_COMBO (cat_combo), cat_list);
-  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cat_combo)->entry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (cat_combo)->entry),
+			     FALSE);
   fd = get_file_data_from_gnode (racine->selected_folder);
   if (fd->categorie)
     {
@@ -303,7 +338,9 @@ edit_folder (GtkWidget * w, FOLDER * racine)
 
   GList *desc_list;
   int i;
+  GtkAccelGroup *accel_group;
 
+  accel_group = gtk_accel_group_new ();
 
   if (racine->selected_folder != NULL)
     {
@@ -312,6 +349,7 @@ edit_folder (GtkWidget * w, FOLDER * racine)
       /* Window */
       edit_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
+      gtk_window_set_modal (GTK_WINDOW (edit_window), TRUE);
       gtk_window_set_transient_for (GTK_WINDOW (edit_window),
 				    GTK_WINDOW (main_window));
 
@@ -322,16 +360,36 @@ edit_folder (GtkWidget * w, FOLDER * racine)
       else
 	gtk_widget_set_usize (GTK_WIDGET (edit_window), 300, 210);
       if (is_disk (racine->selected_folder) == TRUE)
-	gtk_window_set_title (GTK_WINDOW (edit_window), "Edit disk");
+	{
+	  gtk_window_set_title (GTK_WINDOW (edit_window), _("Edit disk"));
+	  gtk_window_set_wmclass (GTK_WINDOW (edit_window), _("Edit disk"),
+				  "GTKtalog");
+	}
       else if (is_vfs_any (racine->selected_folder) == TRUE)
-	gtk_window_set_title (GTK_WINDOW (edit_window), "Edit vfs");
+	{
+	  gtk_window_set_title (GTK_WINDOW (edit_window), _("Edit vfs"));
+	  gtk_window_set_wmclass (GTK_WINDOW (edit_window), _("Edit vfs"),
+				  "GTKtalog");
+	}
       else if (is_link (racine->selected_folder) == TRUE)
-	gtk_window_set_title (GTK_WINDOW (edit_window), "Edit link");
+	{
+	  gtk_window_set_title (GTK_WINDOW (edit_window), _("Edit link"));
+	  gtk_window_set_wmclass (GTK_WINDOW (edit_window), _("Edit link"),
+				  "GTKtalog");
+	}
       else if (is_dir (racine->selected_folder) == TRUE)
-	gtk_window_set_title (GTK_WINDOW (edit_window), "Edit folder");
+	{
+	  gtk_window_set_title (GTK_WINDOW (edit_window), _("Edit folder"));
+	  gtk_window_set_wmclass (GTK_WINDOW (edit_window), _("Edit folder"),
+				  "GTKtalog");
+	}
       else if (is_file (racine->selected_folder) == TRUE)
-	gtk_window_set_title (GTK_WINDOW (edit_window), "Edit file");
-      gtk_window_set_policy (GTK_WINDOW (edit_window), FALSE, FALSE, TRUE);
+	{
+	  gtk_window_set_title (GTK_WINDOW (edit_window), _("Edit file"));
+	  gtk_window_set_wmclass (GTK_WINDOW (edit_window), _("Edit file"),
+				  "GTKtalog");
+	}
+      gtk_window_set_policy (GTK_WINDOW (edit_window), TRUE, TRUE, TRUE);
       gtk_container_border_width (GTK_CONTAINER (edit_window), 10);
       gtk_signal_connect (GTK_OBJECT (edit_window),
 			  "delete_event", destroy_window, NULL);
@@ -419,8 +477,8 @@ edit_folder (GtkWidget * w, FOLDER * racine)
 	  gtk_combo_set_popdown_strings (GTK_COMBO (cat_combo), cat_list);
 	  g_list_free (cat_list);
 	}
-      gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cat_combo)->entry),
-			      FALSE);
+      gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (cat_combo)->entry),
+				 FALSE);
       if (fd->categorie)
 	{
 	  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (cat_combo)->entry),
@@ -479,10 +537,12 @@ edit_folder (GtkWidget * w, FOLDER * racine)
       gtk_container_add (GTK_CONTAINER (hboxdc), desc_combo);
       desc_list = NULL;
       for (i = 0; i < racine->descriptions->len; i++)
-	desc_list = g_list_append (desc_list, ((GString *)
-					       g_ptr_array_index
-					       (racine->descriptions,
-						i))->str);
+	{
+	  desc_list = g_list_append (desc_list, ((GString *)
+						 g_ptr_array_index
+						 (racine->descriptions,
+						  i))->str);
+	}
       if (desc_list)
 	gtk_combo_set_popdown_strings (GTK_COMBO (desc_combo), desc_list);
       if (fd->description)
@@ -557,6 +617,10 @@ edit_folder (GtkWidget * w, FOLDER * racine)
 			  NULL);
       gtk_widget_show (cancel);
 
+      gtk_widget_add_accelerator (cancel, "clicked", accel_group,
+				  GDK_Escape, 0, GTK_ACCEL_VISIBLE);
+
+      gtk_window_add_accel_group (GTK_WINDOW (edit_window), accel_group);
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
       gtk_widget_show (hbox);
 
@@ -571,14 +635,56 @@ update_disk (GtkWidget * w, FOLDER * racine)
 {
   GNode *parent;
   gchar *folder_name, *disk_name;
+  const gchar *fn;
 
+  reset_added_disk ();
   parent = racine->selected_folder;
   disk_name = g_strdup (folder_get_name (parent));
-  folder_name = g_strdup (folder_get_informations (parent));
-
-  suppress_dir (racine->selected_folder, TRUE);
-  gtk_clist_clear (GTK_CLIST (clist));
-  start_add_disk (folder_name, disk_name, racine);
-  g_free (folder_name);
+  if (is_cd (parent))
+    {
+      folder_name = g_strdup (my_config->mount_point->str);
+    }
+  else
+    {
+      if ((fn = folder_get_informations (parent)))
+	folder_name = g_strdup (fn);
+      else
+	folder_name = NULL;
+    }
+  if (folder_name)
+    {
+      if (is_cd (parent))
+	{
+	  do_not_forget_to_mount (TRUE);
+	  if (!mount_disk_if_necessary ())
+	    {
+	      g_free (folder_name);
+	      g_free (disk_name);
+	      display_message_if_necessary ();
+	      reinit_thread_status ();
+	      return;
+	    }
+	}
+      suppress_dir (racine->selected_folder, TRUE);
+      gtk_clist_clear (GTK_CLIST (clist));
+      gtk_widget_hide (GTK_WIDGET (clist));
+      gtk_clist_columns_autosize (GTK_CLIST (clist));
+      gtk_widget_show (GTK_WIDGET (clist));
+      if (is_cd (parent))
+	{
+	  start_add_cd_for_update (folder_name, disk_name, racine);
+	}
+      else
+	{
+	  start_add_disk (folder_name, disk_name, racine);
+	}
+      g_free (folder_name);
+    }
+  else
+    {
+      ERROR_DIALOG (_
+		    ("The disk has an invalid path. Report the bug to the authors."),
+		    main_window);
+    }
   g_free (disk_name);
 }
