@@ -62,14 +62,6 @@ get_catalog_from_gnode (GNode * gn)
   return (root_gn->data);
 }
 
-void
-catalog_set_selected_folder (GNode * gn)
-{
-  FOLDER *f;
-  f = get_catalog_from_gnode (gn);
-  f->selected_folder = gn;
-}
-
 void
 catalog_set_currently_displayed_gnode (GNode * gn)
 {
@@ -107,8 +99,15 @@ get_real_path_from_node (GNode * gn)
       tmp = tmp->parent;
     }
   fd = get_file_data_from_gnode (tmp);
-  if (fd->information)
-    rp = g_string_prepend (rp, fd->information->str);
+  if (fd->type == IS_CD)
+    {
+      rp = g_string_prepend (rp, my_config->mount_point->str);
+    }
+  else
+    {
+      fd = get_file_data_from_gnode (tmp);
+      rp = g_string_prepend (rp, fd->information->str);
+    }
   rp = g_string_prepend_c (rp, '/');
 
   return (rp);
@@ -221,6 +220,67 @@ change_name (GNode * gn, char *name)
 #endif
 }
 
+void
+folder_check_and_set_unique_filename (FOLDER * racine, FILE_DATA * fd)
+{
+  GNode *gn = fd->node;
+  GNode *pgn = g_node_first_sibling (gn);
+  GString *name = fd->name;
+  int i = 1;
+  FILE_DATA *pfd;
+
+  while (pgn)
+    {
+      pfd = pgn->data;
+      if (pfd != fd)
+	{
+	  if (!strcmp (pfd->name->str, name->str))
+	    {
+	      if ((is_link (pfd->node)) && (!is_link (gn)))
+		{
+		  folder_check_and_set_unique_filename (racine, pfd);
+		  name = fd->name;
+		}
+	      else
+		{
+
+		  if (name == fd->name)
+		    name = g_string_new (fd->name->str);
+		  i++;
+		  g_string_sprintf (name, "%s (%d)", fd->name->str, i);
+		}
+	      pgn = g_node_first_sibling (gn);
+	    }
+	}
+      pgn = g_node_next_sibling (pgn);
+    }
+  if (i > 1)
+    {
+      g_string_free (fd->name, TRUE);
+      fd->name = name;
+    }
+}
+
+gboolean
+is_disk_name_unique (FOLDER * racine, gchar * temp)
+{
+  GNode *gn = racine->tree;
+  GNode *pgn = g_node_first_child (gn);
+  FILE_DATA *pfd;
+
+  while (pgn)
+    {
+      pfd = pgn->data;
+      if (!strcmp (pfd->name->str, temp))
+	{
+	  return (FALSE);
+	}
+
+      pgn = g_node_next_sibling (pgn);
+    }
+  return (TRUE);
+}
+
 void
 change_description (GNode * gn, guint i)
 {
@@ -292,6 +352,10 @@ const char *
 folder_get_informations (GNode * gn)
 {
   FILE_DATA *fd;
+  static gchar *cd_string = N_("Cdrom");
+
+  if (is_cd (gn))
+    return (cd_string);
 
   fd = get_file_data_from_gnode (gn);
   if (fd->information)
@@ -328,20 +392,36 @@ folder_get_description_from_file_data (FILE_DATA * fd)
 }
 
 char *
-get_time (time_t nb_sec, char *pattern)
+get_time (time_t nb_sec, gint type, char *pattern)
 {
   struct tm *time;
   time_t nb_sec2;
   char *mydate;
   if (nb_sec)
     {
-      if (pattern)
+      if ((pattern) && (type == DATE_FORMAT_IS_USERDEFINED))
 	{
 	  time = localtime (&nb_sec);
 	  mydate = (char *) g_malloc (sizeof (char) * 256);
 	  if (!strftime (mydate, 255, pattern, time))
 	    {
-	      free (mydate);
+	      g_free (mydate);
+	      memcpy (&nb_sec2, &nb_sec, sizeof (time_t));
+	      mydate = g_strdup (ctime (&nb_sec2));
+	    }
+	}
+      else if (type == DATE_FORMAT_IS_LOCALE)
+	{
+	  memcpy (&nb_sec2, &nb_sec, sizeof (time_t));
+	  mydate = g_strdup (ctime (&nb_sec2));
+	}
+      else if (type == DATE_FORMAT_IS_ISO8601)
+	{
+	  time = localtime (&nb_sec);
+	  mydate = (char *) g_malloc (sizeof (char) * 256);
+	  if (!strftime (mydate, 255, "%G-%m-%d %H:%M:%S", time))
+	    {
+	      g_free (mydate);
 	      memcpy (&nb_sec2, &nb_sec, sizeof (time_t));
 	      mydate = g_strdup (ctime (&nb_sec2));
 	    }
@@ -359,15 +439,59 @@ get_time (time_t nb_sec, char *pattern)
   return (mydate);
 }
 
+static char *
+get_size_unit (guint32 nb_octs)
+{
+  if (my_config->size_format == SIZE_FORMAT_GB ||
+      (((my_config->size_format == SIZE_FORMAT_HUMAN) ||
+	(my_config->size_format == SIZE_FORMAT_B_HUMAN)) && (nb_octs >> 30)))
+    return g_strdup_printf ("%.2f %s", (gfloat) nb_octs / (1 << 30), _("Gb"));
+
+  if (my_config->size_format == SIZE_FORMAT_MB ||
+      ((my_config->size_format == SIZE_FORMAT_HUMAN ||
+	(my_config->size_format == SIZE_FORMAT_B_HUMAN)) && (nb_octs >> 20)))
+    return g_strdup_printf ("%.2f %s", (gfloat) nb_octs / (1 << 20), _("Mb"));
+
+  if ((my_config->size_format == SIZE_FORMAT_KB) ||
+      (((my_config->size_format == SIZE_FORMAT_HUMAN) ||
+	my_config->size_format == SIZE_FORMAT_B_HUMAN) && (nb_octs >> 10)))
+    return g_strdup_printf ("%.2f %s", (gfloat) nb_octs / (1 << 10), _("Kb"));
+
+  else
+    return g_strdup_printf ("%u", nb_octs);
+}
 
 char *
 get_size (guint32 nb_octs)
 {
-  char *mytaille;
+  gchar *unit;
+  gchar *ret;
 
-  mytaille = (char *) g_malloc (sizeof (char) * 15);
-  sprintf (mytaille, "%lu", (long unsigned int) nb_octs);
-  return (mytaille);
+  unit = get_size_unit (nb_octs);
+  if ((my_config->size_format == SIZE_FORMAT_B_HUMAN) && (nb_octs >> 10))
+    {
+      ret = g_strdup_printf ("%u (%s)", nb_octs, unit);
+      g_free (unit);
+    }
+  else
+    ret = unit;
+
+  return ret;
+}
+
+char *
+get_size_mb (guint32 nb_mocts)
+{
+  if ((my_config->size_format == SIZE_FORMAT_B_HUMAN) && (nb_mocts >> 10))
+    return g_strdup_printf ("%u %s (%.2f %s)", nb_mocts, _("Mb"),
+			    (gfloat) nb_mocts / (1 << 10), _("Gb"));
+
+  else if (((my_config->size_format == SIZE_FORMAT_HUMAN) && (nb_mocts >> 10))
+	   || my_config->size_format == SIZE_FORMAT_GB)
+    return g_strdup_printf ("%.2f %s", (gfloat) nb_mocts / (1 << 10),
+			    _("Gb"));
+  else
+    return g_strdup_printf ("%u %s", nb_mocts, _("Mb"));
 }
 
 int
@@ -386,8 +510,7 @@ get_categorie (FOLDER * racine, guint16 i)
 {
   if ((i <= racine->categories->len) && (i > 0))
     {
-      return (
-	      ((GString *)
+      return (((GString *)
 	       g_ptr_array_index (racine->categories, i - 1))->str);
     }
   else
@@ -419,8 +542,7 @@ get_description (FOLDER * racine, guint16 i)
 {
   if ((i <= racine->descriptions->len) && (i > 0))
     {
-      return (
-	      ((GString *)
+      return (((GString *)
 	       g_ptr_array_index (racine->descriptions, i - 1))->str);
     }
   else
@@ -481,7 +603,32 @@ is_disk (GNode * gn)
   FILE_DATA *fd;
 
   fd = get_file_data_from_gnode (gn);
-  if (fd->type == IS_DISK)
+  if ((fd->type == IS_DISK) || (fd->type == IS_CD))
+    return (TRUE);
+  return (FALSE);
+}
+
+gboolean
+is_cd (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if (fd->type == IS_CD)
+    return (TRUE);
+  return (FALSE);
+}
+
+gboolean
+is_a_parent_cd (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  GNode *gnp = gn;
+  while (g_node_depth (gnp) > 1)
+    gnp = gnp->parent;
+  fd = get_file_data_from_gnode (gnp);
+  if (fd->type == IS_CD)
     return (TRUE);
   return (FALSE);
 }
@@ -531,6 +678,17 @@ is_in_vfs_any (GNode * gn)
   return (FALSE);
 }
 
+gboolean
+is_link_dest (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if (fd->type == IS_LINK_DEST)
+    return (TRUE);
+  return (FALSE);
+}
+
 /* Find a FOLDER structure f with (f->name == name).
  * already_found sould be NULL
  * recursive is a flag to make a recursive search or only a one-level search
@@ -586,13 +744,13 @@ _gnode_clear_data (GNode * gn, gpointer data)
   return (FALSE);
 }
 
-gboolean _gnode_clear_ctree_data (GNode * gn, gpointer data)
+gboolean
+_gnode_clear_ctree_data (GNode * gn, gpointer data)
 {
   FOLDER *racine = data;
   FILE_DATA *fd;
 
-  if (
-      ((is_dir (gn) == TRUE) || (is_disk (gn) == TRUE)
+  if (((is_dir (gn) == TRUE) || (is_disk (gn) == TRUE)
        || (is_vfs_any (gn) == TRUE)) && (G_NODE_IS_ROOT (gn) != TRUE))
     {
       fd = get_file_data_from_gnode (gn);
@@ -701,6 +859,70 @@ _make_disk_list (GNode * gn, gpointer data)
   return;
 }
 
+gint
+sort_filenames_with_numbers (gchar * text1, gchar * text2)
+{
+  char *p1, *p2;
+  gint n1, n2;
+  gint result;
+
+  if (!text2)
+    return (text1 != NULL);
+
+  if (!text1)
+    return -1;
+
+  if (my_config->strict_alphabetic_sort)
+    return (strcmp (text1, text2));
+
+  result = 0;
+  p1 = text1;
+  p2 = text2;
+  while ((p1[0] != 0) && (p2[0] != 0) && (result == 0))
+    {
+      if ((p1[0] >= '0') && (p1[0] <= '9') && (p2[0] >= '0')
+	  && (p2[0] <= '9'))
+	{
+	  n1 = strtol (p1, NULL, 10);
+	  n2 = strtol (p2, NULL, 10);
+	  if (n1 > n2)
+	    result = 1;
+	  if (n1 < n2)
+	    result = -1;
+	  while ((p1[0] >= '0') && (p1[0] <= '9'))
+	    p1++;
+	  while ((p2[0] >= '0') && (p2[0] <= '9'))
+	    p2++;
+	}
+      else
+	{
+	  result = strncmp (p1, p2, 1);
+	  p1++;
+	  p2++;
+	}
+    }
+  if (result == 0)
+    {
+
+      if (p1[0] && (!p2[0]))
+	result = 1;
+      if (p2[0] && (!p1[0]))
+	result = -1;
+    }
+
+  return (result);
+
+}
+
+gint
+_sort_disk_list (gconstpointer a, gconstpointer b)
+{
+
+  return (sort_filenames_with_numbers ((gchar *) a, (gchar *) b));
+
+}
+
+
 GList *
 make_disk_list (FOLDER * racine)
 {
@@ -710,6 +932,7 @@ make_disk_list (FOLDER * racine)
 
   g_node_children_foreach (racine->tree, G_TRAVERSE_ALL,
 			   _make_disk_list, &disk_list);
+  disk_list.list = g_list_sort (disk_list.list, _sort_disk_list);
   return (disk_list.list);
 }
 
@@ -737,22 +960,30 @@ _test_description (GNode * gn, gpointer data)
   u = fd->description;
   if (u > 0)
     {
-      if (g_tree_lookup ((GTree *) data, GUINT_TO_POINTER (u)) == NULL)
+      gp = g_ptr_array_index (my_config->racine->descriptions, u - 1);
+      if (gp)
 	{
-	  g_ptr_array_add (new_descriptions,
-			   (gp =
-			    g_ptr_array_index (my_config->racine->
-					       descriptions, u - 1)));
-	  g_assert (gp != NULL);
-	  g_ptr_array_index (my_config->racine->descriptions, u - 1) = NULL;
-	  g_tree_insert ((GTree *) data, GUINT_TO_POINTER (u),
-			 GUINT_TO_POINTER (new_descriptions->len));
+	  if (g_tree_lookup ((GTree *) data, GUINT_TO_POINTER (u)) == NULL)
+	    {
+	      g_ptr_array_add (new_descriptions, (gp));
+	      g_assert (gp != NULL);
+	      g_ptr_array_index (my_config->racine->descriptions, u - 1) =
+		NULL;
+	      g_tree_insert ((GTree *) data, GUINT_TO_POINTER (u),
+			     GUINT_TO_POINTER (new_descriptions->len));
+	    }
+	}
+      else
+	{
+	  if (g_tree_lookup ((GTree *) data, GUINT_TO_POINTER (u)) == NULL)
+	    fd->description = 0;
 	}
     }
   return (FALSE);
 }
 
-gboolean _adapt_description (GNode * gn, gpointer data)
+gboolean
+_adapt_description (GNode * gn, gpointer data)
 {
   FILE_DATA *fd;
   guint u = 0, nu;
@@ -778,7 +1009,6 @@ description_garbage_collection ()
   GTree *corresp = g_tree_new (_cmp_intptrs);
   GString *gs;
   gint i;
-
   new_descriptions = g_ptr_array_new ();
   g_node_traverse (racine->tree,
 		   G_PRE_ORDER, G_TRAVERSE_ALL, -1, _test_description,
@@ -830,6 +1060,7 @@ init_folder ()
 
   racine->ctree = NULL;
 
+
   fd->node = racine->tree;
 
 #if defined(__DEBUG__)
@@ -878,7 +1109,8 @@ clear_folder (FOLDER * racine)
 
 }
 
-gboolean _scan_folders_for_stats (GNode * gn, gpointer data)
+gboolean
+_scan_folders_for_stats (GNode * gn, gpointer data)
 {
   guint32 *stats = data;
   FILE_DATA *fd;
@@ -926,15 +1158,15 @@ folder_statistics (FOLDER * racine, GNode * gn, guint32 * nb_desc,
    */
   if (nb_desc)
     {
-      *nb_desc = racine->descriptions->len;
+      *nb_desc += racine->descriptions->len;
     }
   if (nb_cat)
     {
-      *nb_desc = racine->categories->len;
+      *nb_cat += racine->categories->len;
     }
   if (nb_disks)
     {
-      *nb_disks = g_node_n_children (racine->tree);
+      *nb_disks += g_node_n_children (racine->tree);
     }
   if (nb_dirs || nb_dirs_with_vfs || nb_files || nb_files_with_vfs
       || total_size)
@@ -942,18 +1174,34 @@ folder_statistics (FOLDER * racine, GNode * gn, guint32 * nb_desc,
       g_node_traverse (gn, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 		       _scan_folders_for_stats, stats);
       if (nb_dirs)
-	*nb_dirs = stats[2] - stats[0];
+	*nb_dirs += (stats[2] - stats[0]);
       if (nb_dirs_with_vfs)
-	*nb_dirs_with_vfs = stats[2];
+	*nb_dirs_with_vfs += stats[2];
       if (nb_files)
-	*nb_files = stats[3] - stats[1] - 1;
+	{
+	  *nb_files += (stats[3] - stats[1]);
+	  if (G_NODE_IS_ROOT (gn))
+	    *nb_files -= 1;
+	}
       if (nb_files_with_vfs)
-	*nb_files_with_vfs = stats[3] - 1;
+	{
+	  *nb_files_with_vfs += stats[3];
+	  if (G_NODE_IS_ROOT (gn))
+	    *nb_files_with_vfs -= 1;
+	}
       if (total_size)
-	*total_size = stats[4];
+	*total_size += stats[4];
       if (total_size_2)
-	*total_size_2 = stats[5];
+	*total_size_2 += stats[5];
       if (nb_vfs)
-	*nb_vfs = stats[6];
+	*nb_vfs += stats[6];
+      if (total_size_2)
+	{
+	  if (total_size)
+	    {
+	      *total_size += (*total_size_2 / 1048576);
+	      *total_size_2 = (*total_size_2 % 1048576);
+	    }
+	}
     }
 }
