@@ -17,6 +17,13 @@
  */
 
 #include <config.h>
+#ifdef OS_LINUX
+#  include <sys/vfs.h>
+#endif
+#if defined(OS_FREEBSD) || defined(OS_NETBSD) || defined(OS_OPENBSD)
+#    include <sys/param.h>
+#    include <sys/mount.h>
+#endif
 #include <gnome.h>
 #include <strings.h>
 #include <stdio.h>
@@ -35,12 +42,17 @@
 #include "folder.h"
 #include "compare.h"
 #include "progressbar.h"
+#include "thread_utils.h"
 
 
 static GtkWidget *scrolled_window;
 static GtkWidget *ctree_window;
 static GtkWidget *clist_window;
 static GtkWidget *hpaned;
+static GtkWidget *vbox_containing_ctree;
+
+static GNode *current_displayed_gnode = NULL;
+static gboolean current_displayed_gnode_expand_tree;
 
 GtkWidget *clist;
 GtkWidget *path_entry;
@@ -68,7 +80,6 @@ traverse_add_dir_to_clist (GNode * gn, gpointer data)
   PIX *pix;
   guint32 total_size1 = 0;
   guint32 total_size2 = 0;
-  gchar tmpstr[22];
 
   if ((is_dir (gn) == TRUE) || (is_disk (gn) == TRUE)
       || (is_vfs_any (gn) == TRUE))
@@ -89,15 +100,14 @@ traverse_add_dir_to_clist (GNode * gn, gpointer data)
 				 NULL, NULL, NULL,
 				 NULL, NULL, &total_size1, &total_size2,
 				 NULL);
-	      if (total_size1)
+	      if (total_size1 >> 12)	/* 12 = 32 - 20; 32 is for the length of total_size; 20 is for Mb */
 		{
-		  snprintf (tmpstr, 21, "%d Mb", total_size1);
-		  temp[1] = g_strdup (tmpstr);
+		  temp[1] = get_size_mb (total_size1);
 		}
 	      else
 		{
-		  snprintf (tmpstr, 19, "%d", total_size2);
-		  temp[1] = g_strdup (tmpstr);
+		  temp[1] =
+		    get_size ((total_size1 * (1 << 20)) + total_size2);
 		}
 	    }
 	  else
@@ -106,7 +116,9 @@ traverse_add_dir_to_clist (GNode * gn, gpointer data)
       if (fd->a_parent_node_is_vfs == IS_VFS)
 	temp[2] = g_strdup ("-");
       else
-	temp[2] = get_time (fd->date, my_config->date_format);
+	temp[2] =
+	  get_time (fd->date, my_config->date_format_type,
+		    my_config->date_format);
       temp[3] = folder_get_category_from_file_data (fd);
       if ((!temp[3]) || (fd->a_parent_node_is_vfs == IS_VFS))
 	temp[3] = nothing;
@@ -157,7 +169,7 @@ traverse_add_file_to_clist (GNode * gn, gpointer data)
 	}
 
       if ((fd->a_parent_node_is_vfs == IS_VFS)
-	  || (is_unreadable (gn) == TRUE))
+	  || (is_unreadable (gn) == TRUE) || (is_link (gn) == TRUE))
 	temp[1] = g_strdup ("-");
       else
 	temp[1] = get_size (fd->taille);
@@ -165,7 +177,9 @@ traverse_add_file_to_clist (GNode * gn, gpointer data)
 	  || (is_unreadable (gn) == TRUE))
 	temp[2] = g_strdup ("-");
       else
-	temp[2] = get_time (fd->date, my_config->date_format);
+	temp[2] =
+	  get_time (fd->date, my_config->date_format_type,
+		    my_config->date_format);
       temp[3] = folder_get_category_from_file_data (fd);
       if ((!temp[3]) || (fd->a_parent_node_is_vfs == IS_VFS))
 	temp[3] = nothing;
@@ -311,7 +325,7 @@ launch_viewer (GNode * gn, gint row)
 	{
 	  WARNING_DIALOG (_
 			  ("Files inside Virtual File Systems cannot be displayed"),
-main_window);
+			  main_window);
 	  return;
 	}
       if ((my_config->dbclick_opens_info)
@@ -324,11 +338,11 @@ main_window);
       else
 	{
 	  fullname = get_real_path_from_node (gn);
-	  if (g_file_exists (fullname->str) == FALSE)
+	  if (!g_file_test (fullname->str, G_FILE_TEST_ISFILE))
 	    {
 	      ERROR_DIALOG (_
 			    ("The file does not exist.\nMaybe it was removed or it is on a CD.\nIn this case, load the CD, mount it and try again"),
-main_window);
+			    main_window);
 	    }
 	  else
 	    {
@@ -383,7 +397,7 @@ main_window);
 			{
 			  ERROR_DIALOG (_
 					("The filename contains `%f'. This conflicts with the viewer definition."),
-main_window);
+					main_window);
 
 			}
 		    }
@@ -416,7 +430,7 @@ main_window);
 			{
 			  ERROR_DIALOG (_
 					("This file has no viewer, and it cannot be displayed as text."),
-main_window);
+					main_window);
 			}
 		    }
 		}
@@ -454,6 +468,21 @@ fin_fils (int sig)
   signal (SIGCHLD, fin_fils);
 }
 
+void
+cb_select_all (GtkWidget * w, gpointer data)
+{
+  if (clist)
+    gtk_clist_select_all (GTK_CLIST (clist));
+}
+
+void
+cb_select_none (GtkWidget * w, gpointer data)
+{
+  if (clist)
+    gtk_clist_unselect_all (GTK_CLIST (clist));
+}
+
+
 static void
 selection_made (GtkWidget * clist, gint row, gint column,
 		GdkEventButton * event, gpointer data)
@@ -461,19 +490,23 @@ selection_made (GtkWidget * clist, gint row, gint column,
   GNode *gn;
   FILE_DATA *fd;
   FOLDER *racine;
-
-  racine =
-    get_catalog_from_gnode (gtk_clist_get_row_data (GTK_CLIST (clist), row));
-  racine->selected_folder = NULL;
-
-  gtk_widget_grab_focus (GTK_WIDGET (clist));
-
-  fd = get_file_data_from_gnode (racine->tree);
-  gtk_ctree_unselect_recursive (GTK_CTREE (racine->ctree), fd->ctree_node);
   gn = gtk_clist_get_row_data (GTK_CLIST (clist), row);
-  catalog_set_selected_folder (gn);
-  if ((event->button == 1) && (event->type == GDK_2BUTTON_PRESS))
-    launch_viewer (gn, row);
+  if (gn)
+    {
+      racine = get_catalog_from_gnode (gn);
+
+      fd = get_file_data_from_gnode (racine->tree);
+      gtk_ctree_unselect_recursive (GTK_CTREE (racine->ctree),
+				    fd->ctree_node);
+      gn = gtk_clist_get_row_data (GTK_CLIST (clist), row);
+      racine->selected_folder = gn;
+
+      if (event)
+	{
+	  if ((event->button == 1) && (event->type == GDK_2BUTTON_PRESS))
+	    launch_viewer (gn, row);
+	}
+    }
 }
 
 
@@ -482,27 +515,72 @@ unselection_made (GtkWidget * clist, gint row, gint column,
 		  GdkEventButton * event, gpointer data)
 {
   FOLDER *racine;
+  FILE_DATA *fd;
 
   racine =
     get_catalog_from_gnode (gtk_clist_get_row_data (GTK_CLIST (clist), row));
+
+  fd = get_file_data_from_gnode (racine->tree);
   racine->selected_folder = NULL;
 }
 
-void
-rm_tree (GNode * gn, FOLDER * racine)
-{
-  FILE_DATA *fd = get_file_data_from_gnode (gn);
-
-  gtk_ctree_remove_node (GTK_CTREE (racine->ctree), fd->ctree_node);
-}
-
-gboolean set_vfs_fd (GNode * gn, gpointer data)
+gboolean
+set_vfs_fd (GNode * gn, gpointer data)
 {
   FILE_DATA *fd = get_file_data_from_gnode (gn);
   fd->a_parent_node_is_vfs = IS_VFS;
   return (FALSE);
 }
 
+void
+change_vfs_to_file (FOLDER * racine, GNode * gn, GNode * parent)
+{
+  FILE_DATA *fd = gn->data;
+
+  GString *bak_name;		/* name of the file/dir */
+  guint32 bak_taille;		/* length of the file/dir */
+  time_t bak_date;		/* time_t */
+  GString *bak_mime;		/* Mime type of the file */
+  guint16 bak_categorie;
+  guint16 bak_description;
+  GString *bak_information;
+
+  bak_name = g_string_new (fd->name->str);
+  bak_taille = fd->taille;
+  bak_date = fd->date;
+  if (fd->information)
+    {
+      bak_mime = g_string_new (fd->mime->str);
+    }
+  else
+    {
+      bak_mime = NULL;
+    }
+  bak_categorie = fd->categorie;
+  bak_description = fd->description;
+  if (fd->information)
+    {
+      bak_information = g_string_new (fd->information->str);
+    }
+  else
+    {
+      bak_information = NULL;
+    }
+  gtk_clist_freeze (GTK_CLIST (racine->ctree));
+  suppress_dir (fd->node, TRUE);
+  gn =
+    folder_add (racine, parent, bak_name->str,
+		bak_information->str, IS_FILE, bak_mime->str,
+		bak_taille, bak_date, bak_categorie, bak_description);
+  fd = gn->data;
+  racine->selected_folder = gn;
+  add_gnode_to_ctree (fd->node, racine);
+  gtk_clist_thaw (GTK_CLIST (racine->ctree));
+  g_string_free (bak_name, TRUE);
+  g_string_free (bak_mime, TRUE);
+  g_string_free (bak_information, TRUE);
+}
+
 void
 edit_vfs_mode (GtkWidget * w, FOLDER * racine)
 {
@@ -510,6 +588,7 @@ edit_vfs_mode (GtkWidget * w, FOLDER * racine)
   GtkWidget *dialog, *label;
   FILE_DATA *fd;
 
+
   if (racine->selected_folder != NULL)
     {
 
@@ -530,17 +609,21 @@ edit_vfs_mode (GtkWidget * w, FOLDER * racine)
 			      TRUE, TRUE, 0);
 	  gtk_widget_show (label);
 	  returned_value = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+	  while (gtk_events_pending ())
+	    gtk_main_iteration ();
 	  switch (returned_value)
 	    {
 	    case 0:
 	      g_node_traverse (fd->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 			       set_vfs_fd, NULL);
 	      list_folder (racine->selected_folder->parent, TRUE);
+	      my_config->racine->is_modified = TRUE;
 	      break;
 	    case 1:
-	      change_type (fd->node, IS_FILE);
-	      rm_tree (fd->node, racine);
+	      change_vfs_to_file (racine, fd->node,
+				  racine->selected_folder->parent);
 	      list_folder (racine->selected_folder->parent, TRUE);
+	      my_config->racine->is_modified = TRUE;
 	      break;
 	    }
 	}
@@ -559,15 +642,22 @@ edit_vfs_mode (GtkWidget * w, FOLDER * racine)
 	  returned_value = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 	  if (returned_value == 0)
 	    {
-	      change_type (fd->node, IS_FILE);
-	      rm_tree (fd->node, racine);
+	      change_vfs_to_file (racine, fd->node,
+				  racine->selected_folder->parent);
 	      list_folder (racine->selected_folder->parent, TRUE);
+	      my_config->racine->is_modified = TRUE;
 	    }
 	}
     }
 
 }
 
+GnomeUIInfo clist_pressed_menu_multi_dir[] = {
+  GNOMEUIINFO_ITEM_NONE (N_("Delete selected files/dirs"), NULL,
+			 delete_folder),
+  GNOMEUIINFO_END
+};
+
 GnomeUIInfo tree_pressed_menu_dir[] = {
   GNOMEUIINFO_ITEM_NONE (N_("Edit directory"), NULL, edit_folder),
   GNOMEUIINFO_ITEM_NONE (N_("Delete directory"), NULL, delete_folder),
@@ -598,8 +688,6 @@ GnomeUIInfo tree_pressed_menu_disk[] = {
 
 GnomeUIInfo clist_pressed_menu_file[] = {
   GNOMEUIINFO_ITEM_NONE (N_("Edit file"), NULL, edit_folder),
-  /*  GNOMEUIINFO_ITEM_NONE (N_("Edit information"), NULL,
-   * edit_file_informations), */
   GNOMEUIINFO_ITEM_NONE (N_("Delete file"), NULL, delete_folder),
   GNOMEUIINFO_END
 };
@@ -608,7 +696,12 @@ gint
 cmp_2file_types (GNode * gn1, GNode * gn2)
 {
   if (G_NODE_IS_ROOT (gn1->parent))
-    return (-1);
+    {
+      if (G_NODE_IS_ROOT (gn2->parent))
+	return (0);
+      else
+	return (-1);
+    }
   if (G_NODE_IS_ROOT (gn2->parent))
     return (+1);
 
@@ -659,7 +752,7 @@ clist_cmp_2files (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
   fd1 = get_file_data_from_gnode (gn1);
   fd2 = get_file_data_from_gnode (gn2);
 
-  cmp = strcmp (fd1->name->str, fd2->name->str);
+  cmp = sort_filenames_with_numbers (fd1->name->str, fd2->name->str);
   return (cmp);
 }
 
@@ -786,7 +879,8 @@ set_currently_displayed_folder_pixtext (FOLDER * racine, gboolean is_open)
 				      NULL, NULL);
 	  gtk_ctree_node_set_pixtext (GTK_CTREE (racine->ctree),
 				      fd->ctree_node, 0,
-				      g_strdup(dir_text), 4, pix->pixmap, *(pix->mask));
+				      g_strdup (dir_text), 4, pix->pixmap,
+				      *(pix->mask));
 	  gn = gn->parent;
 
 	}
@@ -804,9 +898,18 @@ list_folder (GNode * parent, gboolean expand_tree)
   int i;
   /*  selected_clist_folder = NULL; */
 
+  if (parent == NULL)
+    {
+      parent = current_displayed_gnode;
+      expand_tree = current_displayed_gnode_expand_tree;
+    }
+  current_displayed_gnode = parent;
+  current_displayed_gnode_expand_tree = expand_tree;
+  if (current_displayed_gnode == NULL)
+    return;
+
   gtk_clist_freeze (GTK_CLIST (clist));
   gtk_clist_clear (GTK_CLIST (clist));
-
   if (g_node_depth (parent) > 1)
     {
       fd = get_file_data_from_gnode (parent->parent);
@@ -819,7 +922,9 @@ list_folder (GNode * parent, gboolean expand_tree)
       if (fd->a_parent_node_is_vfs == IS_VFS)
 	temp[2] = g_strdup ("-");
       else
-	temp[2] = get_time (fd->date, my_config->date_format);
+	temp[2] =
+	  get_time (fd->date, my_config->date_format_type,
+		    my_config->date_format);
       temp[3] = folder_get_category_from_file_data (fd);
       if ((!temp[3]) || (fd->a_parent_node_is_vfs == IS_VFS))
 	temp[3] = nothing;
@@ -830,7 +935,9 @@ list_folder (GNode * parent, gboolean expand_tree)
       if ((!temp[5]) || (fd->a_parent_node_is_vfs == IS_VFS))
 	temp[5] = nothing;
 
+
       i = gtk_clist_append (GTK_CLIST (clist), (gchar **) temp);
+      gtk_clist_set_selectable (GTK_CLIST (clist), i, FALSE);
 
       if (g_node_depth (parent) == 2)
 	pix = my_config->default_disk_pix;
@@ -842,7 +949,6 @@ list_folder (GNode * parent, gboolean expand_tree)
       gtk_clist_set_pixtext (GTK_CLIST (clist), i, 0,
 			     temp[0], 3, pix->pixmap, *(pix->mask));
       gtk_clist_set_row_data (GTK_CLIST (clist), i, parent);
-
       g_free ((gpointer) temp[0]);
       g_free ((gpointer) temp[1]);
       g_free ((gpointer) temp[2]);
@@ -869,9 +975,19 @@ list_folder (GNode * parent, gboolean expand_tree)
 
   if (expand_tree)
     expand_folder (parent);
+
+  gtk_clist_columns_autosize (GTK_CLIST (clist));
 }
 
-
+void
+key_pressed (GtkWidget * wid, GdkEventKey * event, gpointer data)
+{
+  if ((event->keyval == GDK_Delete) || (event->keyval == GDK_KP_Delete))
+    {
+      if (my_config->racine && my_config->racine->selected_folder)
+	delete_folder (wid,my_config->racine);
+    }
+}
 
 void
 clist_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data)
@@ -884,48 +1000,57 @@ clist_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data)
 
   if (event->button == 3)
     {
-      x = event->x;
-      y = event->y;
-      if (gtk_clist_get_selection_info
-	  (GTK_CLIST (clist), x, y, &row, &column))
+      if ((get_thread_status () == THREAD_SCAN_RUNNING_NO)
+	  && (!is_search_running ()))
 	{
-	  gn = gtk_clist_get_row_data (GTK_CLIST (clist), row);
-	  catalog_set_selected_folder (gn);
-	  racine = get_catalog_from_gnode (gn);
+	  x = event->x;
+	  y = event->y;
+	  if (gtk_clist_get_selection_info
+	      (GTK_CLIST (wid), x, y, &row, &column))
+	    {
+	      gn = gtk_clist_get_row_data (GTK_CLIST (wid), row);
+	      racine = get_catalog_from_gnode (gn);
+	      racine->selected_folder = gn;
 
-	  if (is_dir (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_dir);
+	      if (g_list_length ((GTK_CLIST (wid))->selection) > 1)
+		{
+		  list_menu =
+		    gnome_popup_menu_new (clist_pressed_menu_multi_dir);
+		}
+	      else if (is_dir (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_dir);
 
+		}
+	      else if (is_link (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_link);
+		}
+	      else if (is_file (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (clist_pressed_menu_file);
+		}
+	      else if (is_unreadable (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (clist_pressed_menu_file);
+		}
+	      else if (is_vfs_any (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_vfs);
+		}
+	      else
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_disk);
+		}
+	      gnome_popup_menu_do_popup (list_menu, NULL, NULL, event,
+					 racine);
 	    }
-	  else if (is_link (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_link);
-	    }
-	  else if (is_file (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (clist_pressed_menu_file);
-	    }
-	  else if (is_unreadable (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (clist_pressed_menu_file);
-	    }
-	  else if (is_vfs_any (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_vfs);
-	    }
-	  else
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_disk);
-	    }
-	  gnome_popup_menu_do_popup (list_menu, NULL, NULL, event, racine);
 	}
     }
 }
 
 
 
-
 GtkWidget *
 create_list ()
 {
@@ -942,19 +1067,16 @@ create_list ()
   clist = gtk_clist_new_with_titles (6, titles);
 
   gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_OUT);
-  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 135);
-  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 60);
-  gtk_clist_set_column_width (GTK_CLIST (clist), 2, 56);
-  gtk_clist_set_column_width (GTK_CLIST (clist), 3, 60);
-  gtk_clist_set_column_width (GTK_CLIST (clist), 4, 60);
   gtk_container_add (GTK_CONTAINER (scrolled_window), clist);
-  gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 5, TRUE);
   gtk_signal_connect (GTK_OBJECT (clist), "button_press_event",
 		      GTK_SIGNAL_FUNC (clist_pressed), NULL);
+  gtk_signal_connect (GTK_OBJECT (clist), "key_press_event",
+		      GTK_SIGNAL_FUNC (key_pressed), NULL);
   gtk_signal_connect (GTK_OBJECT (clist), "select_row",
 		      GTK_SIGNAL_FUNC (selection_made), NULL);
   gtk_signal_connect (GTK_OBJECT (clist), "unselect_row",
 		      GTK_SIGNAL_FUNC (unselection_made), NULL);
+  gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_EXTENDED);
   gtk_widget_show (clist);
 
   return scrolled_window;
@@ -973,32 +1095,38 @@ tree_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data)
 
   if (event->button == 3)
     {
-      x = event->x;
-      y = event->y;
-      if (gtk_clist_get_selection_info (GTK_CLIST (wid), x, y, &row, &column))
+      if ((get_thread_status () == THREAD_SCAN_RUNNING_NO)
+	  && (!is_search_running ()))
 	{
-	  gn = gtk_clist_get_row_data (GTK_CLIST (wid), row);
-	  catalog_set_selected_folder (gn);
-	  racine = get_catalog_from_gnode (gn);
+	  x = event->x;
+	  y = event->y;
+	  if (gtk_clist_get_selection_info
+	      (GTK_CLIST (wid), x, y, &row, &column))
+	    {
+	      gn = gtk_clist_get_row_data (GTK_CLIST (wid), row);
+	      racine = get_catalog_from_gnode (gn);
+	      racine->selected_folder = gn;
 
-	  if (is_dir (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_dir);
+	      if (is_dir (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_dir);
 
+		}
+	      else if (is_link (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_link);
+		}
+	      else if (is_vfs_any (gn) == TRUE)
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_vfs);
+		}
+	      else
+		{
+		  list_menu = gnome_popup_menu_new (tree_pressed_menu_disk);
+		}
+	      gnome_popup_menu_do_popup (list_menu, NULL, NULL, event,
+					 racine);
 	    }
-	  else if (is_link (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_link);
-	    }
-	  else if (is_vfs_any (gn) == TRUE)
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_vfs);
-	    }
-	  else
-	    {
-	      list_menu = gnome_popup_menu_new (tree_pressed_menu_disk);
-	    }
-	  gnome_popup_menu_do_popup (list_menu, NULL, NULL, event, racine);
 	}
     }
   else if ((event->button == 1))
@@ -1008,7 +1136,6 @@ tree_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data)
       if (gtk_clist_get_selection_info (GTK_CLIST (wid), x, y, &row, &column))
 	{
 	  gn = gtk_clist_get_row_data (GTK_CLIST (wid), row);
-	  catalog_set_selected_folder (gn);
 	  racine = get_catalog_from_gnode (gn);
 	  if (G_NODE_IS_ROOT (gn) != TRUE)
 	    {
@@ -1071,33 +1198,55 @@ GnomeUIInfo recent_files_menu[] = {
 };
 
 GnomeUIInfo file_menu[] = {
-  GNOMEUIINFO_MENU_NEW_ITEM (N_("New catalog"), N_("New catalog"),
+  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New catalog"), N_("_New catalog"),
 			     cb_new_file,
 			     NULL),
   GNOMEUIINFO_MENU_OPEN_ITEM (cb_open_cat, NULL),
+  {GNOME_APP_UI_ITEM, N_("_Merge..."), NULL, cb_merge_cat, NULL, NULL,
+   GNOME_APP_PIXMAP_NONE, NULL,
+   GDK_M, (GdkModifierType) GDK_CONTROL_MASK, NULL},
   GNOMEUIINFO_MENU_SAVE_ITEM (cb_fast_save, NULL),
   GNOMEUIINFO_MENU_SAVE_AS_ITEM (cb_save_cat, NULL),
   GNOMEUIINFO_SEPARATOR,
-  GNOMEUIINFO_SUBTREE (N_("Recent files"), recent_files_menu),
+  GNOMEUIINFO_SUBTREE (N_("_Recent files"), recent_files_menu),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_EXIT_ITEM (cb_quit_prog, NULL),
   GNOMEUIINFO_END
 };
 
 GnomeUIInfo edit_menu[] = {
-  GNOMEUIINFO_ITEM_NONE (N_("Edit categories"), NULL, cb_edit_categories),
-  GNOMEUIINFO_ITEM_NONE (N_("Catalog information"), NULL, cb_cat_info),
+  {GNOME_APP_UI_ITEM, N_("_Select all"), NULL, cb_select_all, NULL, NULL,
+   GNOME_APP_PIXMAP_NONE, NULL,
+   GDK_A, (GdkModifierType) GDK_CONTROL_MASK, NULL},
+  GNOMEUIINFO_ITEM_NONE (N_("_Unselect all"), NULL, cb_select_none),
   GNOMEUIINFO_SEPARATOR,
+  {GNOME_APP_UI_ITEM, N_("_Find file"), NULL, cb_create_search_dialog, NULL,
+   NULL, GNOME_APP_PIXMAP_NONE, NULL,
+   GDK_F, (GdkModifierType) GDK_CONTROL_MASK, NULL},
+  GNOMEUIINFO_END
+};
+
+GnomeUIInfo prefs_menu[] = {
   GNOMEUIINFO_MENU_PREFERENCES_ITEM (cb_run_setup, NULL),
   GNOMEUIINFO_END
 };
 
 GnomeUIInfo action_menu[] = {
-  GNOMEUIINFO_ITEM_NONE (N_("Find file"), NULL, cb_create_search_dialog),
-  GNOMEUIINFO_ITEM_NONE (N_("Create Report"), NULL, cb_open_report_dialog),
+  {GNOME_APP_UI_ITEM, N_("_Edit categories"), NULL, cb_edit_categories, NULL,
+   NULL, GNOME_APP_PIXMAP_NONE, NULL,
+   GDK_E, (GdkModifierType) GDK_CONTROL_MASK, NULL},
+  {GNOME_APP_UI_ITEM, N_("Catalog _information"), NULL, cb_cat_info, NULL,
+   NULL, GNOME_APP_PIXMAP_NONE, NULL,
+   GDK_I, (GdkModifierType) GDK_CONTROL_MASK, NULL},
   GNOMEUIINFO_SEPARATOR,
-  GNOMEUIINFO_ITEM_NONE (N_("Add a new CD"), NULL, cb_fast_add_disk),
-  GNOMEUIINFO_ITEM_NONE (N_("Add a new directory"), NULL,
+  {GNOME_APP_UI_ITEM, N_("Create _Report"), NULL, cb_open_report_dialog, NULL,
+   NULL, GNOME_APP_PIXMAP_NONE, NULL,
+   GDK_R, (GdkModifierType) GDK_CONTROL_MASK, NULL},
+  GNOMEUIINFO_SEPARATOR,
+  {GNOME_APP_UI_ITEM, N_("Add a new _CD"), NULL, cb_fast_add_disk, NULL, NULL,
+   GNOME_APP_PIXMAP_NONE, NULL,
+   GDK_C, (GdkModifierType) GDK_CONTROL_MASK, NULL},
+  GNOMEUIINFO_ITEM_NONE (N_("Add a new _directory"), NULL,
 			 cb_show_select_dir_win),
   GNOMEUIINFO_END
 };
@@ -1111,7 +1260,8 @@ GnomeUIInfo help_menu[] = {
 GnomeUIInfo menubar[] = {
   GNOMEUIINFO_MENU_FILE_TREE (file_menu),
   GNOMEUIINFO_MENU_EDIT_TREE (edit_menu),
-  GNOMEUIINFO_SUBTREE (N_("Action"), action_menu),
+  GNOMEUIINFO_SUBTREE (N_("Catalog"), action_menu),
+  GNOMEUIINFO_MENU_SETTINGS_TREE (prefs_menu),
   GNOMEUIINFO_MENU_HELP_TREE (help_menu),
   GNOMEUIINFO_END
 };
@@ -1124,7 +1274,7 @@ _gnome_add_file_menu (gchar * txt, gpointer data)
     GNOMEUIINFO_END
   };
   gchar *path =
-    g_strconcat (GNOME_MENU_FILE_PATH, _("Recent files"), "/", NULL);
+    g_strconcat (GNOME_MENU_FILE_PATH, _("_Recent files"), "/", NULL);
   gnome_app_insert_menus (GNOME_APP (main_window), path, sm);
   g_free (path);
 }
@@ -1133,7 +1283,7 @@ void
 empty_recentfile_menu ()
 {
   gchar *path =
-    g_strconcat (GNOME_MENU_FILE_PATH, _("Recent files"), "/", NULL);
+    g_strconcat (GNOME_MENU_FILE_PATH, _("_Recent files"), "/", NULL);
 
   if (my_config->currently_displayed_recent_files)
     gnome_app_remove_menus (GNOME_APP (main_window), path,
@@ -1147,7 +1297,7 @@ void
 fill_recentfile_menu ()
 {
   gchar *path =
-    g_strconcat (GNOME_MENU_FILE_PATH, _("Recent files"), "/", NULL);
+    g_strconcat (GNOME_MENU_FILE_PATH, _("_Recent files"), "/", NULL);
   RECENT_FILE *rf;
   GList *gl;
 
@@ -1172,10 +1322,13 @@ add_file_to_recent_files (GList * recent_files, gint nb_max_recent_files,
   RECENT_FILE *rf;
   GList *local_recent_files = recent_files;
   GList *gl;
-  gchar *label;
+  gchar *label, *new_label;
   gchar *fullname;
+  gchar *p1, *p2;
+  gint len;
+  gboolean have_underscores;
 
-  fullname = g_strdup(_fullname);
+  fullname = g_strdup (_fullname);
   if (fullname[0] != '/')
     label = g_strdup (fullname);
   else
@@ -1189,8 +1342,42 @@ add_file_to_recent_files (GList * recent_files, gint nb_max_recent_files,
       label = g_strdup (label);
     }
 
-  gl = local_recent_files;
+  /* if the label contains some underscore chars, duplicate them! */
+  len = 1;
+  have_underscores = FALSE;
+  p1 = label;
+  while (p1[0])
+    {
+      if (p1[0] == '_')
+	{
+	  len++;
+	  have_underscores = TRUE;
+	}
+      len++;
+      p1++;
+    }
+  if (have_underscores)
+    {
+      new_label = p2 = (gchar *) g_malloc (sizeof (gchar) * len);
+      p1 = label;
+      while (p1[0])
+	{
+	  if (p1[0] == '_')
+	    {
+	      p2[0] = '_';
+	      p2++;
+	    }
+	  p2[0] = p1[0];
+	  p1++;
+	  p2++;
+	}
+      p2[0] = 0;
+      g_free (label);
+      label = new_label;
+    }
+
   /* Search for existing same entries */
+  gl = local_recent_files;
   while (gl)
     {
       rf = gl->data;
@@ -1234,7 +1421,6 @@ void
 add_file_menu (gchar * fullname)
 {
 #if defined(__DEBUG__)
-  GML_set_mode_active ();
   GML_log_point (1, "add_file_menu");
 #endif
   empty_recentfile_menu ();
@@ -1242,9 +1428,6 @@ add_file_menu (gchar * fullname)
     add_file_to_recent_files (my_config->recent_files,
 			      my_config->nb_max_recent_files, fullname);
   fill_recentfile_menu ();
-#if defined(__DEBUG__)
-  GML_set_mode_passive ();
-#endif
 }
 
 void
@@ -1252,29 +1435,73 @@ menus_set_sensitive (gboolean sensitive)
 {
   gtk_widget_set_sensitive (GTK_WIDGET (file_menu[2].widget), sensitive);
   gtk_widget_set_sensitive (GTK_WIDGET (file_menu[3].widget), sensitive);
+  gtk_widget_set_sensitive (GTK_WIDGET (file_menu[4].widget), sensitive);
   gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[0].widget), sensitive);
   gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[1].widget), sensitive);
+  gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[3].widget), sensitive);
   gtk_widget_set_sensitive (GTK_WIDGET (action_menu[0].widget), sensitive);
   gtk_widget_set_sensitive (GTK_WIDGET (action_menu[1].widget), sensitive);
+  gtk_widget_set_sensitive (GTK_WIDGET (action_menu[3].widget), sensitive);
   buttons_set_sensitive (sensitive);
 }
 
+void
+menus_set_all_sensitive (gboolean sensitive)
+{
+  int i;
+  for (i = 0; i < (sizeof (file_menu) / sizeof (GnomeUIInfo)); i++)
+    {
+      if (file_menu[i].widget)
+	gtk_widget_set_sensitive (GTK_WIDGET (file_menu[i].widget),
+				  sensitive);
+    }
+  for (i = 0; i < (sizeof (edit_menu) / sizeof (GnomeUIInfo)); i++)
+    {
+      if (edit_menu[i].widget)
+	gtk_widget_set_sensitive (GTK_WIDGET (edit_menu[i].widget),
+				  sensitive);
+    }
+  for (i = 0; i < (sizeof (action_menu) / sizeof (GnomeUIInfo)); i++)
+    {
+      if (action_menu[i].widget)
+	gtk_widget_set_sensitive (GTK_WIDGET (action_menu[i].widget),
+				  sensitive);
+    }
+  for (i = 0; i < (sizeof (prefs_menu) / sizeof (GnomeUIInfo)); i++)
+    {
+      if (prefs_menu[i].widget)
+	gtk_widget_set_sensitive (GTK_WIDGET (prefs_menu[i].widget),
+				  sensitive);
+    }
+  buttons_set_all_sensitive (sensitive);
+}
+
+void
+save_ctree_window_size (void)
+{
+  if (my_config->racine)
+    {
+      gdk_window_get_size (my_config->racine->ctree->window,
+			   &(my_config->ctreewindow_w),
+			   &(my_config->ctreewindow_h));
+    }
+}
+
 void
 build_gui (void)
 {
-  GtkWidget *main_vbox;
   GtkWidget *appbar1;
 
 /* Main vertical box */
-  main_vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
-  gnome_app_set_contents (GNOME_APP (main_window), main_vbox);
-  gtk_widget_show (main_vbox);
+  vbox_containing_ctree = gtk_vbox_new (FALSE, 1);
+  gtk_container_border_width (GTK_CONTAINER (vbox_containing_ctree), 1);
+  gnome_app_set_contents (GNOME_APP (main_window), vbox_containing_ctree);
+  gtk_widget_show (vbox_containing_ctree);
 
 /* Menus and button bar */
 
   gnome_app_create_menus_with_data (GNOME_APP (main_window), menubar, NULL);
-  enable_gnome_button_bar (main_window, NULL);
+  enable_gnome_button_bar (main_window, my_config->toolbar_type);
   gtk_menu_item_right_justify (GTK_MENU_ITEM (menubar[3].widget));
   menus_set_sensitive (FALSE);
   fill_recentfile_menu ();
@@ -1286,22 +1513,23 @@ build_gui (void)
 			    (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (appbar1);
   gnome_app_set_statusbar (GNOME_APP (main_window), appbar1);
-  progress_setGnomeAppBar (appbar1, GNOMEAPPBAR_MAIN);
+  progress_setGnomeAppBar (appbar1, GNOMEAPPBAR_MAIN, TRUE);
 
 /* Path entry */
   path_entry = gtk_entry_new_with_max_length (300);
-  gtk_entry_set_editable (GTK_ENTRY (path_entry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (path_entry), FALSE);
   gtk_entry_set_text (GTK_ENTRY (path_entry), "");
   gtk_entry_append_text (GTK_ENTRY (path_entry), "");
   gtk_entry_select_region (GTK_ENTRY (path_entry),
 			   0, GTK_ENTRY (path_entry)->text_length);
-  gtk_box_pack_start (GTK_BOX (main_vbox), path_entry, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox_containing_ctree), path_entry, FALSE,
+		      TRUE, 0);
   gtk_widget_show (path_entry);
 
 
 /* Horizontal pan for the tree and file display */
   hpaned = gtk_hpaned_new ();
-  gtk_container_add (GTK_CONTAINER (main_vbox), hpaned);
+  gtk_container_add (GTK_CONTAINER (vbox_containing_ctree), hpaned);
   gtk_paned_set_handle_size (GTK_PANED (hpaned), 10);
   gtk_paned_set_gutter_size (GTK_PANED (hpaned), 10);
   gtk_widget_show (hpaned);
@@ -1315,7 +1543,11 @@ build_gui (void)
   ctree_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ctree_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_widget_set_usize (ctree_window, 150, 150);
+  if (my_config->ctreewindow_save_wh)
+    gtk_widget_set_usize (ctree_window, my_config->ctreewindow_w,
+			  my_config->ctreewindow_h);
+  else
+    gtk_widget_set_usize (ctree_window, 150, 150);
 
   gtk_widget_show (ctree_window);
 
@@ -1467,7 +1699,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
   snprintf (tmpstr, 19, "%d", total_size);
   gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-  gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
   y2--;
 
   if (gn == racine->tree)
@@ -1478,7 +1710,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 			(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
       snprintf (tmpstr, 19, "%d", nb_desc);
       gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-      gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+      gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
       y2--;
     }
 
@@ -1490,7 +1722,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 			(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
       snprintf (tmpstr, 19, "%d", nb_cat);
       gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-      gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+      gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
       y2--;
     }
 
@@ -1500,7 +1732,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
   snprintf (tmpstr, 19, "%d", nb_files_with_vfs);
   gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-  gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
   y2--;
 
   myentry = gtk_entry_new ();
@@ -1509,7 +1741,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
   snprintf (tmpstr, 19, "%d", nb_dirs_with_vfs);
   gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-  gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
   y2--;
 
   myentry = gtk_entry_new ();
@@ -1518,7 +1750,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
   snprintf (tmpstr, 19, "%d", nb_vfs);
   gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-  gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
   y2--;
 
   myentry = gtk_entry_new ();
@@ -1527,7 +1759,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
   snprintf (tmpstr, 19, "%d", nb_files);
   gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-  gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
   y2--;
 
   myentry = gtk_entry_new ();
@@ -1536,7 +1768,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 		    (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
   snprintf (tmpstr, 19, "%d", nb_dirs);
   gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-  gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+  gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
   y2--;
 
   if (gn == racine->tree)
@@ -1547,7 +1779,7 @@ show_statistics (FOLDER * racine, GNode * gn)
 			(GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
       snprintf (tmpstr, 19, "%d", nb_disks);
       gtk_entry_set_text (GTK_ENTRY (myentry), tmpstr);
-      gtk_entry_set_editable (GTK_ENTRY (myentry), FALSE);
+      gtk_editable_set_editable (GTK_EDITABLE (myentry), FALSE);
     }
   gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 }
