@@ -27,11 +27,14 @@ void ctree_window_add (GtkWidget * ctree);
 void traverse_add_dir_to_clist (GNode * gn, gpointer data);
 void catalog_set_selected_folder (GNode * gn);
 void traverse_add_file_to_clist (GNode * gn, gpointer data);
-gint clist_cmp_2gint (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2);
-gint clist_cmp_2dates (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2);
-void set_currently_displayed_folder_pixtext (FOLDER * racine, gboolean is_open);
+gint clist_cmp_2gint (GtkCList * clist, gconstpointer ptr1,
+		      gconstpointer ptr2);
+gint clist_cmp_2dates (GtkCList * clist, gconstpointer ptr1,
+		       gconstpointer ptr2);
+void set_currently_displayed_folder_pixtext (FOLDER * racine,
+					     gboolean is_open);
 void list_folder (GNode * parent, gboolean expand_tree);
-MIME_EXTENSIONS *find_extension (const gchar *mime);
+MIME_EXTENSIONS *find_extension (const gchar * mime);
 gchar *viewer_command_line (gchar * filename, const gchar * command);
 void fin_fils (int sig);
 void clist_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data);
@@ -39,11 +42,14 @@ GtkWidget *create_list ();
 void tree_pressed (GtkWidget * wid, GdkEventButton * event, gpointer data);
 void update_tree (FOLDER * racine);
 void clean_description (FOLDER * racine);
-GList *add_file_to_recent_files (GList * recent_files, gint nb_max_recent_files, gchar * fullname);
+GList *add_file_to_recent_files (GList * recent_files,
+				 gint nb_max_recent_files, gchar * fullname);
 void add_file_menu (gchar * fullname);
 void menus_set_sensitive (gboolean sensitive);
+void menus_set_all_sensitive (gboolean sensitive);
+void save_ctree_window_size (void);
 void build_gui (void);
 void stats_folder (GtkWidget * w, FOLDER * racine);
-void show_statistics (FOLDER * racine, GNode *gn);
+void show_statistics (FOLDER * racine, GNode * gn);
 
 #endif
