@@ -22,6 +22,7 @@
 #include "exit.h"
 #include "io.h"
 #include "folder.h"
+#include "thread_utils.h"
 
 static GtkWidget *fs;
 static GtkWidget *toggle_save_cat_mode;
@@ -132,24 +133,18 @@ cb_itemsignal (GtkWidget * widget, GtkCTreeNode * node, gint z,
 	       FOLDER * racine)
 {
   GNode *gnode_data;
+
   gnode_data = gtk_ctree_node_get_row_data (GTK_CTREE (widget), node);
   if (G_NODE_IS_ROOT (gnode_data) != TRUE)
     {
       racine->selected_folder = gnode_data;
-
     }
   else
     {
       racine->selected_folder = NULL;
     }
-}
-
-/* quand un repertoire est deselectionne dans l'arbre */
-static void
-cb_itemsignal2 (GtkWidget * widget, GtkCTreeNode * node, gint i,
-		FOLDER * racine)
-{
-  racine->selected_folder = NULL;
+  if (clist)
+    gtk_clist_unselect_all (GTK_CLIST (clist));
 }
 
 static gboolean
@@ -173,9 +168,6 @@ _clist_g_strnatcmp (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
 {
   char *text1 = NULL;
   char *text2 = NULL;
-  char *p1, *p2;
-  gint n1, n2;
-  gint result;
 
   GtkCListRow *row1 = (GtkCListRow *) ptr1;
   GtkCListRow *row2 = (GtkCListRow *) ptr2;
@@ -203,45 +195,7 @@ _clist_g_strnatcmp (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
     default:
       break;
     }
-
-  if (!text2)
-    return (text1 != NULL);
-
-  if (!text1)
-    return -1;
-
-  result = 0;
-  p1 = text1;
-  p2 = text2;
-  while ((p1[0] != 0) && (p1[0] != 0) && (result == 0))
-    {
-      if ((p1[0] >= '0') && (p1[0] <= '9') && (p2[0] >= '0')
-	  && (p2[0] <= '9'))
-	{
-	  n1 = strtol (p1, NULL, 10);
-	  n2 = strtol (p2, NULL, 10);
-	  if (n1 > n2)
-	    result = 1;
-	  if (n1 < n2)
-	    result = -1;
-	  while ((p1[0] >= '0') && (p1[0] <= '9'))
-	    p1++;
-	  while ((p2[0] >= '0') && (p2[0] <= '9'))
-	    p2++;
-	}
-      else
-	{
-	  result = strncmp (p1, p2, 1);
-	  p1++;
-	  p2++;
-	}
-    }
-  if (p1[0] && (!p2[0]))
-    result = -1;
-  if (p2[0] && (!p1[0]))
-    result = 1;
-
-  return (result);
+  return (sort_filenames_with_numbers (text1, text2));
 }
 
 void
@@ -287,8 +241,6 @@ create_tree (FOLDER * racine)
 			    GTK_SIGNAL_FUNC (expand_cb), racine);
   gtk_signal_connect_after (GTK_OBJECT (racine->ctree), "tree_select_row",
 			    GTK_SIGNAL_FUNC (cb_itemsignal), racine);
-  gtk_signal_connect_after (GTK_OBJECT (racine->ctree), "tree_unselect_row",
-			    GTK_SIGNAL_FUNC (cb_itemsignal2), racine);
 
   gtk_object_set_user_data (GTK_OBJECT (racine->ctree), racine->tree);
 
@@ -331,19 +283,22 @@ new_file ()
       ctree_window_add (my_config->racine->ctree);
       update_tree (my_config->racine);
       menus_set_sensitive (TRUE);
+      gtk_window_set_title (GTK_WINDOW (main_window), "GTKtalog V " VERSION);
     }
 
 }
 
 /*
- * ouverture d'un catalogue 
+ * Open a catalog
  */
 void
 open_cat_ok_sel (GtkWidget * w, gpointer data)
 {
   FOLDER *racine = data;
   gchar *s, *p;
+  gint l;
 
+  set_thread_status_to_running ();
   gtk_clist_clear ((GtkCList *) clist);
 
   if (racine != NULL)
@@ -358,13 +313,19 @@ open_cat_ok_sel (GtkWidget * w, gpointer data)
 
   my_config->racine = racine;
 
-  s = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-  if (load_cat_from_file (s, racine) == -1)
+  s = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
+  gtk_widget_destroy (GTK_WIDGET (fs));
+  while (gtk_events_pending ())
+    gtk_main_iteration ();
+
+  l = load_cat_from_file (s, racine);
+  if (l == -1)
     {
+      reinit_thread_status ();
       return;
     }
   g_free (my_config->working_path);
-  my_config->working_path = g_strdup (s);
+  my_config->working_path = s;
   p = my_config->working_path;
   while (p[0])
     p++;
@@ -372,7 +333,6 @@ open_cat_ok_sel (GtkWidget * w, gpointer data)
     p--;
   p[1] = 0;
 
-  gtk_widget_destroy (GTK_WIDGET (fs));
   g_node_traverse (racine->tree,
 		   G_PRE_ORDER, G_TRAVERSE_ALL, -1, add_gnode_to_ctree,
 		   racine);
@@ -382,6 +342,63 @@ open_cat_ok_sel (GtkWidget * w, gpointer data)
   gtk_window_set_title (GTK_WINDOW (main_window),
 			racine->catalog_filename->str);
   menus_set_sensitive (TRUE);
+  reinit_thread_status ();
+  if (l == -2)
+    {
+      WARNING_DIALOG (_
+		      ("At least one disk name was empty. A random name was used instead"),
+		      main_window);
+    }
+}
+
+/*
+ * Merge a catalog
+ */
+void
+merge_cat_ok_sel (GtkWidget * w, gpointer data)
+{
+  FOLDER *racine = data;
+  gchar *s, *p;
+  gint l;
+
+  set_thread_status_to_running ();
+  gtk_clist_clear ((GtkCList *) clist);
+  gtk_clist_clear ((GtkCList *) racine->ctree);
+  gtk_clist_freeze ((GtkCList *) racine->ctree);
+
+  s = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
+  gtk_widget_destroy (GTK_WIDGET (fs));
+  while (gtk_events_pending ())
+    gtk_main_iteration ();
+  l = merge_cat_from_file (s, racine);
+  if (l == -1)
+    {
+      reinit_thread_status ();
+      return;
+    }
+  g_free (my_config->working_path);
+  my_config->working_path = s;
+  p = my_config->working_path;
+  while (p[0])
+    p++;
+  while (p[0] != '/')
+    p--;
+  p[1] = 0;
+
+  g_node_traverse (racine->tree,
+		   G_PRE_ORDER, G_TRAVERSE_ALL, -1, add_gnode_to_ctree,
+		   racine);
+  gtk_clist_thaw ((GtkCList *) racine->ctree);
+  update_tree (racine);
+
+  menus_set_sensitive (TRUE);
+  reinit_thread_status ();
+  if (l == -2)
+    {
+      WARNING_DIALOG (_
+		      ("At least one disk name was empty. A random name was used instead"),
+		      main_window);
+    }
 }
 
 
@@ -389,6 +406,8 @@ open_cat_ok_sel (GtkWidget * w, gpointer data)
 void
 open_cat (FOLDER * racine)
 {
+  GtkAccelGroup *accel_group;
+
   if (racine != NULL)
     {
       if (racine->is_modified == TRUE)
@@ -402,6 +421,8 @@ open_cat (FOLDER * racine)
 	    }
 	}
     }
+
+  accel_group = gtk_accel_group_new ();
   fs = gtk_file_selection_new (_("File selection"));
   gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
 				   my_config->working_path);
@@ -414,17 +435,62 @@ open_cat (FOLDER * racine)
 			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
 			     GTK_OBJECT (fs));
   gtk_widget_show (fs);
+  gtk_widget_add_accelerator ((GTK_FILE_SELECTION (fs)->cancel_button),
+			      "clicked", accel_group, GDK_Escape, 0,
+			      GTK_ACCEL_VISIBLE);
 
+  gtk_window_add_accel_group (GTK_WINDOW (fs), accel_group);
+  gtk_window_set_transient_for (GTK_WINDOW (fs), GTK_WINDOW (main_window));
+  gtk_window_set_modal (GTK_WINDOW (fs), TRUE);
+}
+
+void
+merge_cat (FOLDER * racine)
+{
+  GtkAccelGroup *accel_group;
+  accel_group = gtk_accel_group_new ();
+  fs = gtk_file_selection_new (_("File selection"));
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
+				   my_config->working_path);
+  gtk_signal_connect (GTK_OBJECT (fs), "destroy",
+		      (GtkSignalFunc) gtk_widget_destroy, &(fs));
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
+		      "clicked", (GtkSignalFunc) merge_cat_ok_sel, racine);
+  gtk_signal_connect_object (GTK_OBJECT
+			     (GTK_FILE_SELECTION (fs)->cancel_button),
+			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
+			     GTK_OBJECT (fs));
+  gtk_widget_show (fs);
+  gtk_widget_add_accelerator ((GTK_FILE_SELECTION (fs)->cancel_button),
+			      "clicked", accel_group, GDK_Escape, 0,
+			      GTK_ACCEL_VISIBLE);
+
+  gtk_window_add_accel_group (GTK_WINDOW (fs), accel_group);
+  gtk_window_set_transient_for (GTK_WINDOW (fs), GTK_WINDOW (main_window));
+  gtk_window_set_modal (GTK_WINDOW (fs), TRUE);
 }
 
 /*
- * permet d'enregistrer le catalogue courant dans un fichier 
+ * Write the current catalog to a file. 
  */
 void
 save_cat_ok_sel (GtkWidget * w, FOLDER * racine)
 {
   gchar *s, *p;
   s = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
+  if (!strcmp ("", s))
+    {
+      WARNING_DIALOG (_("Please specify a file name."), main_window);
+      return;
+    }
+  if (g_file_test (s, G_FILE_TEST_ISDIR))
+    {
+      WARNING_DIALOG (_
+		      ("Either you did not specify a file name or the file is already existing as a directory\n"
+		       "Please specify another file name"), main_window);
+      return;
+    }
+
   my_config->save_only_used_categories =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle_save_cat_mode));
   racine->catalog_filename_is_valid = FALSE;
@@ -441,15 +507,26 @@ save_cat_ok_sel (GtkWidget * w, FOLDER * racine)
 	p--;
       p[1] = 0;
     }
+  else
+    {
+      ERROR_DIALOG (_("Can't write file"), main_window);
+    }
   gtk_widget_destroy (GTK_WIDGET (fs));
 }
 
-
+void
+save_cat_cancel (GtkWidget * w, GtkWidget * fs)
+{
+  gtk_widget_destroy (fs);
+  my_config->want_to_exit = WANT_TO_EXIT_CANCELLED;
+}
 
 void
 save_cat (FOLDER * racine)
 {
+  GtkAccelGroup *accel_group;
   GtkWidget *frame;
+  accel_group = gtk_accel_group_new ();
   fs = gtk_file_selection_new (_("Save catalog to file"));
   gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
 				   my_config->working_path);
@@ -469,37 +546,66 @@ save_cat (FOLDER * racine)
 		      (GtkSignalFunc) gtk_widget_destroy, &(fs));
   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
 		      "clicked", (GtkSignalFunc) save_cat_ok_sel, racine);
-  gtk_signal_connect_object (GTK_OBJECT
-			     (GTK_FILE_SELECTION (fs)->cancel_button),
-			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
-			     GTK_OBJECT (fs));
+  gtk_signal_connect (GTK_OBJECT
+		      (GTK_FILE_SELECTION (fs)->cancel_button),
+		      "clicked", (GtkSignalFunc) save_cat_cancel,
+		      GTK_OBJECT (fs));
   gtk_widget_show (fs);
-}
+  gtk_widget_add_accelerator ((GTK_FILE_SELECTION (fs)->cancel_button),
+			      "clicked", accel_group, GDK_Escape, 0,
+			      GTK_ACCEL_VISIBLE);
 
+  gtk_window_add_accel_group (GTK_WINDOW (fs), accel_group);
+  gtk_window_set_transient_for (GTK_WINDOW (fs), GTK_WINDOW (main_window));
+  gtk_window_set_modal (GTK_WINDOW (fs), TRUE);
+}
 
 void
 fast_save (FOLDER * racine)
 {
+  gchar *filename;
+  if (!racine->is_modified)
+    return;
   if (racine->catalog_filename_is_valid == FALSE)
     {
       save_cat (racine);
     }
   else
     {
-      if (save_cat_to_file (racine->catalog_filename->str, racine) == -1)
+      filename = g_strdup (racine->catalog_filename->str);
+      if (save_cat_to_file (filename, racine) == -1)
 	{
 	  /*
 	   * canceled = 1; 
 	   */
 	  ERROR_DIALOG (_("Can't write file"), main_window);
 	}
+      g_free (filename);
     }
 }
 
+gint
+autosave (gpointer data)
+{
+  if ((my_config->racine->is_modified) && my_config->autosave
+      && my_config->racine->catalog_filename_is_valid)
+    {
+      if ((get_thread_status () == THREAD_SCAN_RUNNING_NO)
+	  && (!is_search_running ()) && (!is_saving ()))
+	{
+	  fast_save (my_config->racine);
+	}
+    }
+
+  gtk_timeout_add (60000 * my_config->minutes_before_autosave, autosave,
+		   NULL);
+  return (0);
+}
+
 /* string_to_index(descriptions, "my own description") will return the index where the
  * description containing the string "my own description" can be reached.
  * id = string_to_index(descriptions, "my own description");
- * each FILE_DATA where the field FILE_dATA->descriptions is id contains the description
+ * each FILE_DATA where the field FILE_DATA->descriptions is id contains the description
  * "my own description".
  * This can be used with categories the same way.
  */
@@ -528,6 +634,8 @@ test_plugin_existence (GString * filename)
   gchar *path;
   gchar **paths, **p1;
   gchar *fullname = NULL;
+  int i;
+  
   if (filename->str[0] == '/')
     return (g_file_test (filename->str, G_FILE_TEST_ISFILE));
   path = getenv ("PATH");
@@ -536,6 +644,9 @@ test_plugin_existence (GString * filename)
   while (*p1)
     {
       fullname = g_strconcat (*p1, "/", filename->str, NULL);
+      for(i=0; (fullname[i] != '\0') && (fullname[i] != ' ') && (fullname[i] !=
+      '\t'); i++);
+      fullname[i] = '\0';
       if (g_file_test (fullname, G_FILE_TEST_ISFILE))
 	break;
       g_free (fullname);
