@@ -17,6 +17,10 @@
  */
 
 #include <config.h>
+#if defined(OS_FREEBSD) || defined(OS_NETBSD) || defined(OS_OPENBSD)
+#    include <sys/param.h>
+#    include <sys/mount.h>
+#endif
 #include <gnome.h>
 #include <string.h>
 
@@ -30,7 +34,9 @@
 #include "io.h"
 #include "loadcat.h"
 #include "setup.h"
+#include "splash.h"
 #include "thread_utils.h"
+#include "progressbar.h"
 
 #define P_FILE		100
 #define P_GEOMETRY	101
@@ -43,11 +49,14 @@ static char *popt_filename = NULL;
 static char *popt_geometry = NULL;
 static char *popt_wintitle = NULL;
 struct poptOption command_line[] = {
-  {"catalog", 'c', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &popt_filename, P_FILE,
+  {"catalog", 'c', POPT_ARG_STRING
+   || POPT_ARGFLAG_ONEDASH, &popt_filename, P_FILE,
    N_("Catalog file to load."), "filename"},
-  {"geometry", 'g', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &popt_geometry, P_GEOMETRY,
+  {"geometry", 'g', POPT_ARG_STRING
+   || POPT_ARGFLAG_ONEDASH, &popt_geometry, P_GEOMETRY,
    N_("Geometry. See `man X' for more info.")},
-  {"title", 't', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &popt_wintitle, P_WINTITLE,
+  {"title", 't', POPT_ARG_STRING
+   || POPT_ARGFLAG_ONEDASH, &popt_wintitle, P_WINTITLE,
    N_("Title of the window"), "text"},
   {"version", 'v', POPT_ARGFLAG_ONEDASH, NULL, P_VERSION,
    N_("Prints the version of " PACKAGE)},
@@ -93,7 +102,6 @@ session_die (GnomeClient * client, gpointer client_data)
   quit_prog (my_config->racine);
 }
 
-
 int
 main (int argc, char *argv[])
 {
@@ -105,9 +113,13 @@ main (int argc, char *argv[])
   int c;
   GnomeClient *client;
   gchar *cwd, *fullname_prog;
+  int l;
 
 #if defined(__DEBUG__)
-  GML_init (5, PACKAGE, FALSE);
+  GML_init (5, PACKAGE);
+  GML_init_server (11142);
+  GML_init_log_file (FALSE);
+  GML_log_point (0, "main() debut");
 #endif
   cwd = g_get_current_dir ();
   bindtextdomain (PACKAGE, GNOMELOCALEDIR);
@@ -124,8 +136,7 @@ main (int argc, char *argv[])
 	{
 	case P_VERSION:
 	  printf (_("Gtktalog\nVersion: %s\nLicence: GPL\n"
-		    "More infos on http://gtktalog.sourceforge.net\n"),
-		  VERSION);
+		    "More infos on " GTKTALOGWEB "\n"), VERSION);
 	  exit (0);
 	  break;
 	case P_FILE:
@@ -163,6 +174,8 @@ main (int argc, char *argv[])
     {
       gtk_window_set_title (GTK_WINDOW (main_window), "GTKtalog V " VERSION);
     }
+  gtk_window_set_wmclass (GTK_WINDOW (main_window),
+			  "GTKtalog V " VERSION, "GTKtalog");
   gtk_widget_realize (main_window);
 
   my_config = init_config (GTK_WIDGET (main_window));
@@ -170,7 +183,8 @@ main (int argc, char *argv[])
   my_config->calendar_widgets = NULL;
   my_config->fast_name = NULL;
   my_config->racine = NULL;
-  my_config->working_path = g_strconcat (getenv ("HOME"), "/", NULL);
+
+  splash_screen (GTK_WIDGET (main_window));
 
   if (_geometry)
     {
@@ -203,6 +217,8 @@ main (int argc, char *argv[])
 
   build_gui ();
 
+  progress_setGnomeAppBar (NULL, GNOMEAPPBAR_SCAN, FALSE);
+
   fullname_prog = argv[0];
   if (!g_path_is_absolute (fullname_prog))
     {
@@ -215,6 +231,8 @@ main (int argc, char *argv[])
   gtk_signal_connect (GTK_OBJECT (client), "die",
 		      GTK_SIGNAL_FUNC (session_die), NULL);
 
+  gtk_signal_connect (GTK_OBJECT (main_window), "delete_event", cb_quit_prog,
+		      my_config);
 
   gtk_widget_show (main_window);
   if ((my_config->mainwindow_save_xywh) || (_geometry))
@@ -222,12 +240,18 @@ main (int argc, char *argv[])
 
   my_config->current_pid = getpid ();
 
+  if ((!_filename) && (my_config->load_file_at_boot))
+    {
+      _filename = g_string_new (my_config->file_to_load);
+    }
+
   if (_filename)
     {
+      set_thread_status_to_running ();
       gtk_entry_set_text (GTK_ENTRY (path_entry), "");
       my_config->racine = init_folder ();
       create_tree (my_config->racine);
-      load_cat_from_file (_filename->str, my_config->racine);
+      l = load_cat_from_file (_filename->str, my_config->racine);
       g_node_traverse (my_config->racine->tree,
 		       G_PRE_ORDER, G_TRAVERSE_ALL, -1, add_gnode_to_ctree,
 		       my_config->racine);
@@ -238,22 +262,29 @@ main (int argc, char *argv[])
 			    my_config->racine->catalog_filename->str);
       g_string_free (_filename, TRUE);
       menus_set_sensitive (TRUE);
+      reinit_thread_status ();
+      if (l == -2)
+	{
+	  WARNING_DIALOG (_
+			  ("At least one disk name was empty. A random name was used instead"),
+			  main_window);
+	}
+
     }
 
-  gtk_signal_connect (GTK_OBJECT (main_window), "delete_event", cb_quit_prog,
-		      my_config);
 #if defined(__DEBUG__)
   GML_log_point (5, "main() DUMP END: before gtk_main()");
 #endif
+  set_search_running (FALSE);
+
+
+  gtk_timeout_add (60000 * my_config->minutes_before_autosave, autosave,
+		   NULL);
+
   gtk_main ();
 
 #if defined(__DEBUG__)
   GML_log_point (5, "main(): quitting");
-  GML_set_mode_passive ();
-  GML_dump (GML_tree_gstrings, FALSE);
-  GML_dump (GML_tree_mallocs, FALSE);
-  GML_dump (GML_tree_strings, FALSE);
-  GML_dump (GML_tree_garrays, FALSE);
   GML_end ();
 #endif
   return (0);
