@@ -17,6 +17,10 @@
  */
 
 #include <config.h>
+#if defined(OS_FREEBSD) || defined(OS_NETBSD) || defined(OS_OPENBSD)
+#    include <sys/param.h>
+#    include <sys/mount.h>
+#endif
 #include <gnome.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -34,12 +38,21 @@
 #include "folder.h"
 #include "thread_utils.h"
 #include "progressbar.h"
+#include "interface.h"
 
 static gzFile gzcatalog;
 #if defined(USE_PTHREADS)
 static pthread_t thread1;
 #endif
 
+gboolean saving = FALSE;
+
+gboolean
+is_saving (void)
+{
+  return (saving);
+}
+
 void
 write_gstring (GString * g, int fh)
 {
@@ -106,14 +119,12 @@ write_vfsnode (GNode * gn, gpointer data)
   gchar id = CAT_TREE_0;
   guint le;
   FILE_DATA *fd = get_file_data_from_gnode (gn);
-  gulong pb;
 
   gzwrite (gzcatalog, &id, sizeof (gchar));
   gzwrite_gstring (fd->name, gzcatalog);
   le = GUINT_TO_LE (fd->type);
   gzwrite (gzcatalog, &le, sizeof (guint));
-  pb = progress_getCurrent (GNOMEAPPBAR_MAIN) + 1;
-  progress_setCurrent (pb, GNOMEAPPBAR_MAIN);
+  progress_addCurrent (1, GNOMEAPPBAR_MAIN);
 #if !defined(USE_PTHREADS)
   gui_update ();
 #endif
@@ -128,7 +139,6 @@ write_filedata (FILE_DATA * d, gzFile fh)
   guint16 le16, le16_category, le16_description;
   guint16 mask;
   gchar id;
-  gulong pb;
 
   g_assert (d != NULL);
   gzwrite_gstring (d->name, fh);
@@ -181,8 +191,7 @@ write_filedata (FILE_DATA * d, gzFile fh)
       if (d->mime->len)
 	gzwrite_gstring (d->mime, fh);
     }
-  pb = progress_getCurrent (GNOMEAPPBAR_MAIN) + 1;
-  progress_setCurrent (pb, GNOMEAPPBAR_MAIN);
+  progress_addCurrent (1, GNOMEAPPBAR_MAIN);
 #if !defined(USE_PTHREADS)
   gui_update ();
 #endif
@@ -242,14 +251,13 @@ _save_cat_to_file_ (void *arg)
   int catalog = sa->catalog;
   FOLDER *racine = sa->racine;
   char *filename = sa->filename;
-  gulong pb;
   guint16 mask;
   guint16 le16;
 
   /* First, remove unused categories if the user asked for it */
   if (my_config->save_only_used_categories)
     {
-      remove_unused_categories (racine);
+      remove_unused_categories (racine, FALSE);
     }
 
   /* Header */
@@ -260,7 +268,7 @@ _save_cat_to_file_ (void *arg)
   /* reopen with zlib */
   g_assert ((my_config->compression_level >= 0)
 	    && (my_config->compression_level <= 9));
-  sprintf (mode, "wb%d", my_config->compression_level);
+  snprintf (mode, 4, "wb%d", my_config->compression_level);
   gzcatalog = gzdopen (catalog, mode);
 
 /* Write the mask.
@@ -283,8 +291,7 @@ _save_cat_to_file_ (void *arg)
 	{
 	  gzwrite_gstring (g_ptr_array_index (racine->descriptions, i),
 			   gzcatalog);
-	  pb = progress_getCurrent (GNOMEAPPBAR_MAIN) + 1;
-	  progress_setCurrent (pb, GNOMEAPPBAR_MAIN);
+	  progress_addCurrent (1, GNOMEAPPBAR_MAIN);
 #if !defined(USE_PTHREADS)
 	  gui_update ();
 #endif
@@ -301,8 +308,7 @@ _save_cat_to_file_ (void *arg)
 	{
 	  gzwrite_gstring (g_ptr_array_index (racine->categories, i),
 			   gzcatalog);
-	  pb = progress_getCurrent (GNOMEAPPBAR_MAIN) + 1;
-	  progress_setCurrent (pb, GNOMEAPPBAR_MAIN);
+	  progress_addCurrent (1, GNOMEAPPBAR_MAIN);
 #if !defined(USE_PTHREADS)
 	  gui_update ();
 #endif
@@ -318,19 +324,35 @@ _save_cat_to_file_ (void *arg)
   chmod (filename, (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH));
 
   racine->is_modified = FALSE;
-  if (racine->catalog_filename)
-    g_string_free (racine->catalog_filename, TRUE);
-  racine->catalog_filename = g_string_new (filename);
-  racine->catalog_filename_is_valid = TRUE;
   g_free (filename);
 
   set_thread_status_to_ended ();
+  my_config->want_to_exit = WANT_TO_EXIT_SAVED;
 #if defined(USE_PTHREADS)
   pthread_exit (NULL);
 #endif
   return (NULL);
 }
 
+void
+backup (gchar * filename)
+{
+  gchar *filename_bak;
+  if ((!my_config->backupfile_is_saved) && (my_config->keep_bakup_file))
+    {
+      filename_bak = g_strconcat (filename, ".bak", NULL);
+      if ((g_file_test (filename_bak, G_FILE_TEST_ISFILE)) == TRUE)
+	{
+	  unlink (filename_bak);
+	}
+      rename (filename, filename_bak);
+    }
+  else
+    {
+      unlink (filename);
+    }
+}
+
 int
 save_cat_to_file (char *filename, FOLDER * racine)
 {
@@ -339,7 +361,7 @@ save_cat_to_file (char *filename, FOLDER * racine)
   guint32 nb_desc = 0, nb_cat = 0;
   guint32 nb_disks = 0;
 
-
+  saving = TRUE;
   if (racine->catalog_filename_is_valid == FALSE)
     {
       if ((g_file_test (filename, G_FILE_TEST_ISFILE)) == TRUE)
@@ -347,29 +369,37 @@ save_cat_to_file (char *filename, FOLDER * racine)
 	  if (yes_no_dialog
 	      (_("File exists"),
 	       _("File already exists.\nOverwrite it?")) != 0)
-	    return (-1);
+	    {
+	      saving = FALSE;
+	      return (-1);
+	    }
 	  else
-	    unlink (filename);
+	    backup (filename);
 	}
     }
   else
     {
       if ((g_file_test (filename, G_FILE_TEST_ISFILE)) == TRUE)
-	unlink (filename);
+	backup (filename);
     }
 
-  if (
-      (_save_args_.catalog =
-       open (filename, O_CREAT | O_WRONLY | O_TRUNC),
-       (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
+  if (racine->catalog_filename)
+    g_string_free (racine->catalog_filename, TRUE);
+  racine->catalog_filename = g_string_new (filename);
+  racine->catalog_filename_is_valid = TRUE;
+
+  if ((_save_args_.catalog =
+       open (filename, O_CREAT | O_WRONLY | O_TRUNC,
+	     S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
     {
+      saving = FALSE;
       return (-1);
     }
 
   folder_statistics (racine, racine->tree, &nb_desc, &nb_cat,
 		     &nb_disks, NULL, &nb_dirs, NULL, &nb_files, NULL, NULL,
 		     NULL);
-  progress_setTotal (nb_desc + nb_cat + nb_dirs + nb_files + nb_disks,
+  progress_setTotal (0, nb_desc + nb_cat + nb_dirs + nb_files + nb_disks,
 		     GNOMEAPPBAR_MAIN);
 
   set_thread_status_to_running ();
@@ -387,11 +417,16 @@ save_cat_to_file (char *filename, FOLDER * racine)
     }
   else
     {
-      thread_gtk_timeout_with_progress_bar (GINT_TO_POINTER(GNOMEAPPBAR_MAIN));
+      thread_gtk_timeout_with_progress_bar (GINT_TO_POINTER
+					    (GNOMEAPPBAR_MAIN));
     }
 #else
-  thread_gtk_timeout_with_progress_bar (GINT_TO_POINTER(GNOMEAPPBAR_MAIN));
+  thread_gtk_timeout_with_progress_bar (GINT_TO_POINTER (GNOMEAPPBAR_MAIN));
   _save_cat_to_file_ (&_save_args_);
 #endif
+/* Add the filename to the 'recent files' submenu */
+  add_file_menu (filename);
+  saving = FALSE;
+
   return (0);
 }
