@@ -21,6 +21,10 @@
  */
 
 #include <config.h>
+#if defined(OS_FREEBSD) || defined(OS_NETBSD) || defined(OS_OPENBSD)
+#    include <sys/param.h>
+#    include <sys/mount.h>
+#endif
 #include <gnome.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,6 +37,7 @@
 #include "addisk.h"
 #include "selectdir.h"
 #include "progressbar.h"
+#include "folder.h"
 #include "thread_utils.h"
 
 static GtkWidget *fs;
@@ -56,11 +61,10 @@ start_add_disk (char *foldername, char *diskname, FOLDER * racine)
 
   do_not_forget_to_mount (FALSE);
   set_thread_status_to_running ();
-  progress_timeout (NULL);
+  progress_createTimeout (GINT_TO_POINTER (GNOMEAPPBAR_MAIN));
   do_not_forget_to_umount (FALSE);
   init_vfs_scan_status ();
   set_added_disk (ADDED_DISK_OK);
-  fad_wait_window ();
 #if defined(USE_PTHREADS)
   if (pthread_create (&thread1, NULL, start_thread_scan, racine) < 0)
     {
@@ -71,9 +75,11 @@ start_add_disk (char *foldername, char *diskname, FOLDER * racine)
   else
     {
       my_timeout (racine);
+      fad_wait_window ();
     }
 #else
   my_timeout (racine);
+  fad_wait_window ();
   start_thread_scan (racine);
 #endif
   return (0);
@@ -114,6 +120,12 @@ ok_clicked (GtkWidget * widget, gpointer data)
       ERROR_DIALOG (_("You have to write the disk label"), main_window);
       return;
     }
+  else if (!is_disk_name_unique (racine, disk_label))
+    {
+      ERROR_DIALOG (_("This label already exists"), main_window);
+      return;
+    }
+
   else
     {
       /*      if (folder_exist_in(&racine,gtk_entry_get_text(GTK_ENTRY(entry)), my_config->search_dirtree) == 1) return; */
@@ -139,6 +151,10 @@ show_select_dir_win (FOLDER * racine)
   GtkWidget *frame, *vbox;
   GtkWidget *readvfs_menu, *readvfs_menuitem;
 
+  GtkAccelGroup *accel_group;
+
+  accel_group = gtk_accel_group_new ();
+
   fs = gtk_file_selection_new (_("Select Disk location"));
   gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
 				   my_config->working_path);
@@ -213,5 +229,10 @@ show_select_dir_win (FOLDER * racine)
 			     (GTK_FILE_SELECTION (fs)->cancel_button),
 			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (fs));
+  gtk_widget_add_accelerator ((GTK_FILE_SELECTION (fs)->cancel_button),
+			      "clicked", accel_group, GDK_Escape, 0,
+			      GTK_ACCEL_VISIBLE);
+
+  gtk_window_add_accel_group (GTK_WINDOW (fs), accel_group);
   gtk_widget_show (fs);
 }
