@@ -7,10 +7,12 @@
 #if !defined(USE_PTHREADS)
 void gui_update (void);
 #endif
-gint get_thread_status ();
-void reinit_thread_status ();
-void set_thread_status_to_running ();
-void set_thread_status_to_ended ();
+gboolean is_search_running (void);
+void set_search_running (gboolean i);
+gint get_thread_status (void);
+void reinit_thread_status (void);
+void set_thread_status_to_running (void);
+void set_thread_status_to_ended (void);
 gint thread_gtk_timeout_with_progress_bar (gpointer data);
 
 #endif
